#!/usr/bin/python3
'''
OpenQP : Question Bank Management System
Authors  : Ajith Kumar B.P (bpajith@gmail.com)
           T K Muraleedharan (muraleedharantk@kannuruniv.ac.in)
License : GNU GPL version 3
Last edit : 17-Mar-2025

Connects to the database. Changes working directory to 'backup'. Saves the table coursedetails to an Excel file. Scans through all the available courses, creates subdirectory for each, saves the data to an Excel file, saves the figures also in the same directory.
'''

import pandas as pd
import xlsxwriter
import psycopg2,sys,os, shutil
from os import path
from psycopg2 import OperationalError

def readConfig(): # Get the database details from config.txt
	global myhost, myport, mydb
	try:
		f = open('config.txt')
		while 1:
			ss = f.readline().strip()
			if ss == '':
				 break
			if ss[0] == '#':		#comments
				 continue
			item = ss.split(':')
			if item[0] == 'name':
				myname = item[1]
			elif item[0] == 'host':
				myhost = item[1]
			elif item[0] == 'port':
				myport = int(item[1])
			elif item[0] == 'dbname':
				mydb = item[1]
	except:
		print('Config file not found')
		sys.exit()        
	print(myhost, myport, mydb)

readConfig()	

backDir = mydb+'-backup'
pw = 'rea123'

if not os.path.exists(backDir):
    os.mkdir(backDir)
os.chdir(backDir)	

conn = psycopg2.connect(host=myhost,database=mydb,port=myport,user='reader',password=pw)
cursor = conn.cursor()
sql =  "select syllabusYear, inst, code, code2, code3 ,degree, semester, texLanguage, qpmodel, duration, nmodules, faculty, name from coursedetails order by code;"
cursor.execute(sql)
res = cursor.fetchall()
nc = len(res)
print ('Connected. Fetched %d Courses.\n'%nc)
df = pd.DataFrame(columns=['syllYear','Inst','code1','code2','code3','degree','semester','language','qpType','duration','nMods','faculty','name'])

# make the coursedetails backup first
NR = len(res)
for k in range(NR):
    row = res[k]
    #print(row)
    syllYear = row[0]
    Inst = row[1]
    code1 = row[2]
    code2 = row[3]
    code3 = row[4]
    degree = row[5]
    semester = row[6]
    language = row[7]
    qpType = row[8]
    duration = row[9]
    nMods = row[10]
    faculty = row[11]
    name = row[12]
    df.loc[k] =[syllYear,Inst,code1,code2,code3,degree,semester,language,qpType,duration,nMods,faculty,name]
 
writer = pd.ExcelWriter('coursedetails.xlsx', engine='xlsxwriter')
df.to_excel(writer, sheet_name='Sheet1', index=False)
writer._save()	

courses = []
for k in res:
    courses.append(k[2])  # course code

tnq = 0
for cc in courses:
    if os.path.exists(cc):
        shutil.rmtree(cc)	
    os.mkdir(cc)
    os.chdir(cc)
    df = pd.DataFrame(columns=['modnum','unitnum','qtype','mark','level','outcome','author','qtext','atext'])
    #Get the figures first
    try:
        sql = "SELECT figname,figure FROM %s where figname is not null"%cc
        cursor.execute(sql)	
        res = cursor.fetchall()
        for k in res: 
            fname = k[0]		
            figure = k[1]
            f = open(fname,'wb')
            f.write(figure)
            f.close()
    except OperationalError as e:
        print('Import figure Failed: %s'%e)
        sys.exit()
    
    sql = "SELECT modnum, unitnum, qtype, mark, level, outcome,author,qtext,atext FROM %s where figname is null order by modnum,qtype"%cc
    cursor.execute(sql)
    res = cursor.fetchall()
    NR = len(res)
    for k in range(NR):
        row = res[k]
        if row[4].lower == 'inactive':      # discard inactive ones
            continue
        #print(row)
        df.loc[k] =[row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7], row[8]]
        
    writer = pd.ExcelWriter(cc + '.xlsx', engine='xlsxwriter')
    df.to_excel(writer, sheet_name='Sheet1', index=False)
    workbook = writer.book
    
    wrap_format = writer.book.add_format({'text_wrap': True})	
    header2 = workbook.add_format({
    'align':    'left',
    'valign':   'top',
    'font_size': 12,
    'text_wrap': True
        })

    worksheet = writer.sheets['Sheet1']
    worksheet.set_column(0,0,7)
    worksheet.set_column(1,1,8)
    worksheet.set_column(2,2,5)
    worksheet.set_column(4,4,12)
    worksheet.set_column(6,6,20)
    worksheet.set_column('H:I', 60,header2)
    rows, columns = df.shape
    for k in range(rows):
        qt = df.iloc[k]['qtext']
        hq = (1 + len(qt)//60 + qt.count('\n')) * 15
        at = df.iloc[k]['atext']
        ha = (1 + len(at)//60 + qt.count('\n')) * 15
        h = hq if (hq>ha) else ha
        #print(k, hq, ha, h) 
        worksheet.set_row(k+1, h, header2)
    writer._save()	
    tnq += rows
    print(cc,rows)
    os.chdir('..')
print(tnq)
