#!/usr/bin/python3
'''
OpenQP : Question Bank Management System
Authors  : Ajith Kumar B.P (bpajith@gmail.com)
           T K Muraleedharan (muraleedharantk@kannuruniv.ac.in)
License : GNU GPL version 3
Last edit : 10-May-2025

Connects to the database, scans through all the subdirectories (they should be available courseCodes) and uploads the data from Excel files and image files.
'''

import psycopg2,sys, csv,os, getpass
import pandas as pd
import openpyxl
import numpy as np
from pathlib import Path

MAXMOD = 8		    # KU has 8, others upto 5
myport = 5432		# postges default port

if len(sys.argv) != 3:
	print ('Usage: python3 restore-all-XL.py hostname databasename')
	sys.exit(0)
myhost = sys.argv[1]
mydb = sys.argv[2]
myuser = mydb.lower()
pw = getpass.getpass('Enter Database password : ')
	
backDir = mydb + '-backup'
if not os.path.exists(backDir):
	print("Directory '%s' does not exist")
	sys.exit()
os.chdir(backDir)

def putFigures(tex,modnum):
	fragments = tex.split('\\includegraphics')	#search included figures
	figlist = []
	nfig = len(fragments)
	for k in range(1,nfig):
		s1 = fragments[k]
		s2 = s1.split('{')[1]
		figname = s2.split('}')[0]
		if os.path.exists(figname) == False:
			GUI.dialog_critical('File %s NOT found.'%figname)
			return				
		try:
			drawing = open(figname, 'rb').read()
			img = psycopg2.Binary(drawing)
			sql = "insert into %s(modnum,figname, figure) values (%d,'%s', %s);"%(courseName, modnum,figname, img)
			cursor.execute(sql)
		except:
			cursor.execute('rollback;')			
			print('Figure upload Error ', courseName, figname)
			return			
	return True		

#---------------------------------------------------	
def processFile(path):
	try:    
		wb = openpyxl.load_workbook(path)
		sheet = wb['Sheet1']
		rows = list(sheet.iter_rows(values_only=True))
	except: 
		print('File loading Error',path)
		return			   					

	line = 0
	datalist = []
	# Iterate over the rows and columns and replace None with an empty string
	for j in range(1, len(rows)):  # Start from row 1 (ignoring headers)
		row = list(rows[j])
		line+=1
		if len(row) != 9:
			print('Row %d. Input should have 9 items: %s'%(line,str(row)))
			return
		for cellpos in range(len(row)):
			if row[cellpos] is None:  # Check for missing (None) values
				#print(j,cellpos)
				row[cellpos] = ''  # Replace None with an empty string
		try:
			modnum = int(row[0])
			unitrel = row[1]
			qtype = row[2].strip()
			mark = int(row[3])
			level = row[4].strip().capitalize()
			outcome = row[5].strip()
			author = row[6].strip()
			question = row[7].strip()
			answer = row[8].strip()
		except:
			print(f'Parse Error in Row {line}\n')
			return
		datalist.append([modnum, unitrel, qtype, mark, level, outcome, author, question, answer])

	# Add serial numbers
	qna = {'A':[0]*MAXMOD, 'B':[0]*MAXMOD, 'C':[0]*MAXMOD, 'D':[0]*MAXMOD }   
	count = 0
	for k in range(len(datalist)):
		row = datalist[k]
		modnum = row[0]
		qtype = row[2]
		#print(count+1,modnum,qtype)
		qna[qtype][modnum-1] += 1	# generate Question number
		qnum = qna[qtype][modnum-1]
		datalist[k].append(qnum)
   
	courseName = os.path.basename(path).split('.')[0]
	sql = "delete from %s;"%(courseName)
	cursor.execute(sql)
	conn.commit()

	count = 0
	for row in datalist:
		modnum = row[0]
		qtype = row[2]
		sql = "insert into %s (modnum,unitnum,qtype,mark,level, outcome,author, qtext, atext, qnum) values (%d, '%s', '%s', %d, '%s', '%s', '%s', '%s','%s',%d);"%(courseName,row[0],row[1],row[2],row[3],row[4],row[5],row[6].replace("'","''"),row[7].replace("'","''"),row[8].replace("'","''"),row[9])
		cursor.execute(sql)
		
		#upload figures, if any
		fragments = (row[7]+row[8]).split('\\includegraphics')	#search included figures
		nfig = len(fragments)
		for k in range(1,nfig):
			s1 = fragments[k]
			s2 = s1.split('{')[1]
			figname = s2.split('}')[0]
			if os.path.exists(figname) == False:
				print('%s: Figure %s NOT found.'%(courseName,figname))
				sys.exit()				
			drawing = open(figname, 'rb').read()
			img = psycopg2.Binary(drawing)
			sql = "insert into %s(modnum,figname, figure) values (%d,'%s', %s);"%(courseName, modnum,figname, img)
			cursor.execute(sql)
		count += 1
	conn.commit()			
	print(courseName, line, count)
	
conn = psycopg2.connect(database=mydb, host=myhost, user=myuser, password=pw, port = myport)
cursor = conn.cursor()

tc = 0
files = Path('.').glob('*')
for fn in files:
	if os.path.isdir(fn) == False:
		continue
	os.chdir(fn)
	fname = str(fn)+'.xlsx'
	processFile(fname)
	os.chdir('..')
	tc += 1
print('uploaded %d courses'%tc)
