#!/usr/bin/python3

'''
OpenQP : Question Bank Management System
Authors  : Ajith Kumar B.P (bpajith@gmail.com)
           T K Muraleedharan (muraleedharantk@kannuruniv.ac.in)
License : GNU GPL version 3
Last edit : 15-May-2025
'''

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
import psycopg2, sys

MAXMOD = 8

CC1 = 0
SY = 1
INST = 2
NAME = 3
FAC = 4
DEG = 5
SEM = 6
LANG = 7
QPTYPE = 8
DUR = 9
NMODS = 10
CC2 = 11
CC3 = 12

class MainWindow(QMainWindow):   # Creating main window class
    codeFilter = ''	
    myname = None
    myhost = None
    mydb = None
    myport = None
    dbpasswd = None
    conn = None
    cursor = None
    courses = None
    courseCode = None
    syllYear = None
    inst = None
    courseName = None
    faculty = None
    degree = None
    semester = None
    language = None
    qpModel = None
    duration = None
    numModules = None
    program = None
    code2 = None
    code3 = None
    moduleMarkString = None

    syllYears = ['2024', '2025','2026']
    institutions = ['AC', 'UD']

    degrees = ['BA', 'BSc', 'MA', 'MSc','MCom','MBA','MCA','MPESS','FYIMP','FYUGP']
    semesters = ['First', 'Second', 'Third', 'Fourth', 'Fifth', 'Sixth', 'Seventh', 'Eighth']
    semesterFilter = ['All']+semesters

    languages = ['english','french', 'hindi', 'sanskrit','malayalam','tamil','kannada','urdu','arabic']

    qpModels = ['ZU70','ZU50','PU50','YU70','YU50','XU70','XU50','AU30','BU30','CU30', 'XE70',  'XE50', 'WE50', 'XE35', 'XE15', 'XE32', 'XE40', 'XE48', 'XE60', 'XE80', 'XU80']

    durations = ['1.5', '2', '2.5', '3']
     
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        palette = QPalette()
        palette.setColor(QPalette.ColorRole(10), QColor(150,175,150)) 
        self.setPalette(palette)	
        self.setGeometry(100, 100, 1150, 700)			
        fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
        fixedfont.setPointSize(10)

        layout = QVBoxLayout()	
        hbox1 = QHBoxLayout()
        layout.addLayout(hbox1)
        hbox2 = QHBoxLayout()
        layout.addLayout(hbox2)
        hbox3 = QHBoxLayout()
        layout.addLayout(hbox3)
        hbox4 = QHBoxLayout()
        layout.addLayout(hbox4)
        hbox5 = QHBoxLayout()
        layout.addLayout(hbox5)
        hbox6 = QHBoxLayout()
        layout.addLayout(hbox6)

        self.dbSelectButton = QPushButton('Select DB')
        hbox1.addWidget(self.dbSelectButton)
        self.dbSelectButton.clicked.connect(self.readConfig)	

        self.dbConnectButton = QPushButton('Connect to DB')
        s = 'Connects to the Database and Lists all the Courses found.\nTo restrict the number of items displayed\nuse the Faculty and Program filters'
        self.dbConnectButton.setToolTip(s)
        hbox1.addWidget(self.dbConnectButton)
        self.dbConnectButton.clicked.connect(self.connectDB)

        hbox1.addWidget(QLabel('Filter->'))	
        self.ccLineEdit = QLineEdit('')		
        self.ccLineEdit.setMaximumWidth(150)
        hbox1.addWidget(self.ccLineEdit)	
        self.ccLineEdit.textChanged.connect(self.setCodeFilter)		
        hbox1.addStretch()

        self.courseButton = QPushButton('Select Course')
        s = 'Select a course from the pull down menu\nYou may delete or modify the selected course.\nYou can also create a new course after editing the courseCode'
        self.courseButton.setToolTip(s)
        self.courseButton.clicked.connect(self.selectCourse)
        hbox1.addWidget(self.courseButton)
        self.courseMenu = QMenu()
        self.courseButton.setMenu(self.courseMenu)
        self.modButton = QPushButton('Modify Selected Course')
        s = 'Modify the selected course.\nCourse code,QP model and numModules cannot be changed.\nLeave the Password field empty if you do not want to modify it'
        self.modButton.setToolTip(s)
        self.modButton.clicked.connect(self.modifyCourse)
        hbox1.addWidget(self.modButton)
        self.courseButton.setMenu(self.courseMenu)
        self.delButton = QPushButton('Delete Selected Course')
        s = 'Delete the selected Course'
        self.delButton.setToolTip(s)
        self.delButton.clicked.connect(self.delCourse)
        hbox1.addWidget(self.delButton)
        hbox1.addStretch()
        self.addButton = QPushButton('Create New Course')
        s = 'Create a New Course using the Information\n from the text Fields and Menus above'
        self.addButton.setToolTip(s)
        self.addButton.clicked.connect(self.addCourse)
        hbox1.addWidget(self.addButton)

        self.qeditor = QPlainTextEdit()
        self.qeditor.setFont(fixedfont)
        self.qeditor.setLineWrapMode(0)
        self.qeditor.setReadOnly(True)			        
        hbox2.addWidget(self.qeditor)
        x = QLabel('To Create a Course Provide all details. To Delete Couse Code is enough')
        x.setStyleSheet("background-color:white")
        x.setAlignment(Qt.AlignCenter)
        hbox3.addWidget(x)

        self.syllYearButton = QPushButton('Syllabus Year')
        hbox4.addWidget(self.syllYearButton)
        m = QMenu()
        for s in self.syllYears:
            m.addAction(s, lambda item=s: self.selectYear(item))
        self.syllYearButton.setMenu(m)

        self.instButton = QPushButton('Institution')
        hbox4.addWidget(self.instButton)
        m = QMenu()
        for s in self.institutions:
            m.addAction(s, lambda item=s: self.selectInst(item))
        self.instButton.setMenu(m)

        self.degreeButton = QPushButton('Degree')
        hbox4.addWidget(self.degreeButton)
        m = QMenu()
        for deg in self.degrees:
            m.addAction(deg, lambda item=deg: self.selectDegree(item))
        self.degreeButton.setMenu(m)

        self.semesterButton = QPushButton('Semester')
        hbox4.addWidget(self.semesterButton)
        m = QMenu()
        for s in self.semesters:
            m.addAction(s, lambda item=s: self.selectSemester(item))
        self.semesterButton.setMenu(m)

        self.languageButton = QPushButton('Language')
        hbox4.addWidget(self.languageButton)
        m = QMenu()
        for s in self.languages:
            m.addAction(s, lambda item=s: self.selectLanguage(item))
        self.languageButton.setMenu(m)
        
        self.qpModelButton = QPushButton('QP Model')
        hbox4.addWidget(self.qpModelButton)
        m = QMenu()
        for s in self.qpModels:
            m.addAction(s, lambda item=s: self.selectQpModel(item))
        self.qpModelButton.setMenu(m)
        
        self.durationButton = QPushButton('Duration')
        hbox4.addWidget(self.durationButton)
        m = QMenu()
        for s in self.durations:
            m.addAction(s, lambda item=s: self.selectDuration(item))
        self.durationButton.setMenu(m)

        spacer = QLabel(' '*34)
        hbox5.addWidget(spacer)
        self.numModulesButton = QPushButton('numModules')
        self.numModulesButton.setVisible(False)
        hbox5.addWidget(self.numModulesButton)
        m = QMenu()
        for k in  range(1,9):m.addAction(str(k), lambda item=str(k): self.selectnumModules(item))
        self.numModulesButton.setMenu(m)

        self.pmLabels = [None]*MAXMOD
        self.pmEdits = [None]*MAXMOD
        for k in range(MAXMOD):
            self.pmLabels[k] = QLabel('M%d Mark->'%(k+1))
            self.pmLabels[k].setVisible(False)
            hbox5.addWidget(self.pmLabels[k])
            self.pmEdits[k] = QLineEdit('0')
            self.pmEdits[k].setMaximumWidth(50)
            self.pmEdits[k].setVisible(False)
            hbox5.addWidget(self.pmEdits[k])
        hbox5.addStretch()

        a = QLabel('Course Code ')
        hbox6.addWidget(a)		
        self.codeLineEdit = QLineEdit('')
        hbox6.addWidget(self.codeLineEdit)
        a = QLabel('Code2')
        hbox6.addWidget(a)		
        self.code2LineEdit = QLineEdit('')
        hbox6.addWidget(self.code2LineEdit)
        a = QLabel('Code3')
        hbox6.addWidget(a)		
        self.code3LineEdit = QLineEdit('')
        hbox6.addWidget(self.code3LineEdit)
        hbox6.addStretch()

        hbox7 = QHBoxLayout()
        hbox7.addWidget(QLabel('Course Name'))		
        self.nameLineEdit = QLineEdit('')
        hbox7.addWidget(self.nameLineEdit)
        hbox7.addWidget(QLabel('Faculty'))		
        self.facLineEdit = QLineEdit('')
        hbox7.addWidget(self.facLineEdit)
        layout.addLayout(hbox7)

        hbox8 = QHBoxLayout()
        layout.addLayout(hbox8)
        hbox8.addWidget(QLabel('Course Password'))		
        self.pass1LineEdit = QLineEdit('')
        self.pass1LineEdit.setMaximumWidth(100)
        self.pass1LineEdit.setEchoMode(QLineEdit.Password)
        hbox8.addWidget(self.pass1LineEdit)
        self.pass2LineEdit = QLineEdit('')
        self.pass2LineEdit.setMaximumWidth(100)
        self.pass2LineEdit.setEchoMode(QLineEdit.Password)
        hbox8.addWidget(self.pass2LineEdit)
        hbox8.addStretch()

        hbox9 = QHBoxLayout()
        layout.addLayout(hbox9)
        hbox9.addStretch()	
        self.webPass1LineEdit = QLineEdit('')
        self.webPass1LineEdit.setMaximumWidth(100)
        self.webPass1LineEdit.setEchoMode(QLineEdit.Password)
        hbox9.addWidget(self.webPass1LineEdit)
        self.webPass2LineEdit = QLineEdit('')
        self.webPass2LineEdit.setMaximumWidth(100)
        self.webPass2LineEdit.setEchoMode(QLineEdit.Password)
        hbox9.addWidget(self.webPass2LineEdit)
        self.webPassButton = QPushButton('Set Question Paper Generator password')
        s = 'Change the password for the Question paper generation'
        self.webPassButton.setToolTip(s)
        self.webPassButton.clicked.connect(self.setWebPassword)
        hbox9.addWidget(self.webPassButton)
    

        hbox10 = QHBoxLayout()
        layout.addLayout(hbox10)		
        self.msg = QLabel('messages')
        hbox10.addWidget(self.msg)
        
        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)  	# making it central widget	
        self.show()	
        # End of __init__

    def readConfig(self): # Get the database details from config.txt
        dialog = QFileDialog()
        path, _ = dialog.getOpenFileName(GUI, "Open Configuration file (config.txt)", "","Text Files(*.txt)")        
        if len(path) == 0:  # user cancelled selection
            return		
        
        try:
            f = open(path)
            while 1:
                ss = f.readline().strip()
                if ss == '':
                     break
                if ss[0] == '#':		#comments
                     continue
                item = ss.split(':')
                if item[0] == 'name':
                    self.myname = item[1]
                elif item[0] == 'host':
                    self.myhost = item[1]
                elif item[0] == 'port':
                    self.myport = int(item[1])
                elif item[0] == 'dbname':
                    self.mydb = item[1]
        except:
            self.dialog_critical('Config file not found')
            return        
        self.setWindowTitle('%s Server.'%self.myname)

    def getPassword(self):
        text, ok = QInputDialog.getText(None, "Authenticate", "Password for Database", QLineEdit.Password)
        if ok and text:
            return text			

    def dialog_critical(self, s):
        dlg = QMessageBox(self)
        dlg.setText(s)
        dlg.setIcon(QMessageBox.Critical)
        dlg.show()			
    
    def connectDB(self):
        if self.myname == None:
            self.dialog_critical('Select the Database')
            return        
        if self.conn != None:
            self.conn.close()
            self.conn = None
        self.cursor = None
        pw = self.getPassword()
        self.qeditor.setPlainText('')	
        if 1:
            print('Connecting to %s Server...'%(self.myname),end =' ')	
            sys.stdout.flush()	
            self.conn = psycopg2.connect(host=self.myhost, database=self.mydb, port=self.myport, user=self.mydb, password=pw)
            self.cursor = self.conn.cursor()
            print('Connected. Reading Course info...', end =' ')	
            sys.stdout.flush()			
        else:
            s = 'Connection to Database %s failed.'%db
            self.dialog_critical(s)		
            return		
        self.setWindowTitle('%s Server.'%self.myname)
        self.updateCourseList()		

    def updateCourseList(self):
        if 1:
            sql =  'select code, syllabusyear, inst, name, faculty, degree, semester, texlanguage, qpmodel, duration, nmodules, code2, code3 from coursedetails order by code;'
            self.cursor.execute(sql)
            result = self.cursor.fetchall()
        else:
            s = 'Fetching Course details from %s Server failed.'%self.myname
            self.dialog_critical(s)		
            return	
        print('Done')
        self.courseList = result		# Store the course list information
        self.displayCourses()
        self.msg.setText('Found %d Courses.'%len(result))

    def setCodeFilter(self):
        self.codeFilter = self.ccLineEdit.text().upper()
        self.displayCourses()
        
    def displayCourses(self):
        if self.courseList == []:
            return
        if self.courseList[0][QPTYPE][1] == 'U': # unequal module marks
            unequal = True	
            txt = '%-22s %-15s %-15s %-8s %-18s %-12s %-8s %-8s %-10s %-25s %-50s\n'%('Course Code','Code2','Code3','QPModel','Modules:Marks','Degree','Semester', 'Duration', 'TexLang','Faculty','Course Name')
        else:
            unequal = False	       
            txt = '%-22s %-15s %-15s %-8s %-6s %-12s %-8s %-8s %-10s %-25s %-50s\n'%('Course Code','Code2','Code3','QPModel','nMods','Degree','Semester', 'Duration', 'TexLang','Faculty','Course Name')
        
        self.courseMenu.clear()
        count = 0
        for row in self.courseList:
            ss = row[CC1] + row[CC2] + row[CC3]
            if self.codeFilter != '' and self.codeFilter not in ss:  	
                continue		
            self.courseMenu.addAction(row[0], lambda item=row[0]: self.selectCourse(item))	
            count += 1						
            if unequal == True:	
                s = '%-22s %-15s %-15s %-8s %-18s %-12s %-8s %-8s %-10s %-25s %-50s\n'%(row[CC1],row[CC2],row[CC3],row[QPTYPE],row[NMODS],row[DEG][:11],row[SEM],row[DUR],row[LANG], row[FAC], row[NAME].strip())			
            else:
                s = '%-22s %-15s %-15s %-8s %-6s %-12s %-8s %-8s %-10s %-25s %-50s\n'%(row[CC1],row[CC2],row[CC3],row[QPTYPE],row[NMODS][0],row[DEG][:11],row[SEM],row[DUR],row[LANG], row[FAC], row[NAME].strip())			
            
            txt += s[:-1] + '\n'  
        self.qeditor.setPlainText(txt)
        self.courseButton.setText('Select Course(%d)'%count)
                     
    def setWebPassword(self):
        if self.cursor == None:
            self.dialog_critical('Not Connected to any Database.')
            return			
        wpw = self.webPass1LineEdit.text()
        if len(wpw) < 6:
            self.dialog_critical('Must have minimum 6 characters.')
            return	
        if wpw != self.webPass2LineEdit.text():
            self.dialog_critical('Passwords not matching.')
            return
        sql = "alter role reader with password '%s';"%wpw
        try:
            self.cursor.execute(sql)	
            self.conn.commit()		
            self.dialog_critical('Web Password changed.')
        except:
            self.dialog_critical('Web Password changing failed.')
        
    def selectYear(self,s):
        self.syllYear = s
        self.syllYearButton.setText(s)

    def selectInst(self,s):
        self.inst = s
        self.instButton.setText(s)
    
    def selectDegree(self,s):
        self.degree = s
        self.degreeButton.setText(s)			

    def selectSemester(self,s):
        self.semester = s
        self.semesterButton.setText(s)

    def selectLanguage(self,s):
        self.language = s
        self.languageButton.setText(s)

    def selectQpModel(self,s):
        self.qpModel = s
        self.qpModelButton.setText(s)
        self.numModulesButton.setVisible(True)
        self.numModulesButton.setText('numModules')
        self.numModules = None		# Reset number of modules
        for k in range(MAXMOD):
            self.pmLabels[k].setVisible(False)
            self.pmEdits[k].setVisible(False)

    def selectDuration(self,s):
        self.duration = s
        self.durationButton.setText(s)

    def selectnumModules(self,s):		# Also controls the mark widgets
        self.numModulesButton.setText(s+' Modules')
        self.numModules = eval(s)
        print('selectnumModules: ',self.numModules)
        for k in range(MAXMOD):			# Hide the mark entry widgets
            self.pmLabels[k].setVisible(False)
            self.pmEdits[k].setVisible(False)
        if self.qpModel[1] == 'E':      # Equal module mark distribution
            self.moduleMarkString = s
            return
        for k in range(self.numModules):	# Show the required mark entry widgets
            self.pmLabels[k].setVisible(True)
            self.pmEdits[k].setVisible(True)
        #self.getModuleMarks()
            
    def getModuleMarks(self):			# make the string from the Widget inputs
        if self.qpModel == None:
            self.dialog_critical('Error: QP Model is NOT selected')
            return
        if self.qpModel[1] == 'E':
            return True
        ss = '%d'%self.numModules
        tm = 0
        try:
            for k in range(self.numModules):
                ss += ':%s'%self.pmEdits[k].text()
                tm += eval(self.pmEdits[k].text())
        except:
            self.dialog_critical('Invalid marks given.')
            return
        n = int(self.qpModel[-2:])	
        if tm != n:
            self.dialog_critical('Section Marks total should be %d'%n)
            return	
        self.moduleMarkString = ss
        return True

    def initModuleMarks(self,ss):
        print(ss, self.qpModel)
        if self.qpModel[1] == 'E':   # Equal module marks
            return
        sss = ss.split(':')	
        self.numModules = eval(sss[0])
        marks = sss[1:]
        if len(marks) != self.numModules:
            self.dialog_critical('numModules & marks NOT proper. %s'%ss)
            return
        print(marks , self.numModules)
        for k in range(self.numModules):
            self.pmEdits[k].setText(marks[k])
            print(k, marks[k])
            
    #----------------------- Select, Add & Delete ---------------------
    #code | syllabusyear | inst | name | faculty | degree | semester | texlanguage | qpmodel | duration | nmodules | coursetype | program | code2 | code3 		
    
    def selectCourse(self, code):
        for c in self.courseList:
            if code == c[CC1]:
                self.courseCode = code
                self.codeLineEdit.setText(c[CC1][8:])
                self.code2LineEdit.setText(c[CC2])
                self.code3LineEdit.setText(c[CC3])
                self.pass1LineEdit.setText('')
                self.pass2LineEdit.setText('')
                self.nameLineEdit.setText(c[NAME])	
                self.syllYear = c[SY]
                self.syllYearButton.setText(self.syllYear)
                self.inst = c[INST]
                self.instButton.setText(self.inst)	
                self.faculty = c[FAC]
                self.facLineEdit.setText(c[FAC])	
                self.degree = c[DEG]
                self.degreeButton.setText(self.degree)
                self.semester = c[SEM]
                self.semesterButton.setText(self.semester)
                self.language = c[LANG]
                self.languageButton.setText(self.language)
                self.qpModel = c[QPTYPE]
                self.qpModelButton.setText(self.qpModel)
                self.selectQpModel(self.qpModel)
                self.duration = c[DUR]
                self.durationButton.setText(self.duration)	
                self.moduleMarkString = c[NMODS]
                np = self.moduleMarkString.split(':')[0]			
                self.numModules = eval(np)
                self.selectnumModules(np)		# control the widgets
                self.numModulesButton.setText('%d Modules'%self.numModules)		
                self.initModuleMarks(self.moduleMarkString)	# Sets the marks on GUI
        self.setWindowTitle('%s Server. Editing %s'%(self.myname, self.courseCode))

    def validate(self):
        if self.cursor == None:
            self.dialog_critical('Not Connected to any Database')
            return False	
        if self.syllYear == None:		# Year & Inst prefixed with courseCode
            self.dialog_critical('Syllabus Year not selected.')
            return False
        if self.inst == None:		# Year & Inst prefixed with courseCode
            self.dialog_critical('Institution type not selected.')
            return False
            
        spchars = ' !#@$%^&*()/\\?'
        code = self.codeLineEdit.text().strip()
        if len(code) < 6:
            self.dialog_critical('Course Code too short or empty.')
            return False
        for c in spchars:
            if c in code:
                self.dialog_critical('Letter %s not allowed in Coursecode.'%c)
                print (len(code))
                return False
                
        code2 = self.code2LineEdit.text().strip()
        if code2 != '':
            if len(code2) < 6:
                self.dialog_critical('Course Code2 too short or empty.')
                return False
            for c in spchars:
                if c in code2:
                    self.dialog_critical('Letter %s not allowed in Coursecode 2.'%c)
                    return False
        self.code2 = code2
        
        code3 = self.code3LineEdit.text().strip()
        if code3 != '':
            if len(code3) < 6:
                self.dialog_critical('Course Code3 too short or empty.')
                return False
            for c in spchars:
                if c in code3:
                    self.dialog_critical('Letter %s not allowed in Coursecode 3.'%c)
                    return False
        self.code3 = code3
                
        name = self.nameLineEdit.text().strip()
        self.courseName = name		
        if len(name.strip()) < 5:
            self.dialog_critical('Course Name too short or not given.')
            self.courseName = None
            return False	

        fac = self.facLineEdit.text().strip()
        if len(fac) < 5:
            self.dialog_critical('Faculty Name too short or not given.')
            self.faculty = None
            return False	
        self.faculty = fac

        if self.degree == None:
            self.dialog_critical('Degree not selected.')
            return False	
        if self.semester == None:
            self.dialog_critical('Semester not selected.')
            return False	
        if self.language == None:
            self.dialog_critical('Type setting language not selected.')
            return False	
        if self.qpModel == None:
            self.dialog_critical('QP Model not selected.')
            return False	
        if self.duration == None:
            self.dialog_critical('Duration not selected.')
            return False	
        if self.numModules == None:
            self.dialog_critical('Number of Modules not selected.')
            return False	
        if self.getModuleMarks() != True:
            self.dialog_critical('Module Marks not obtained.')
            return False	
        
    def modifyCourse(self):	
        if self.courseCode == None:
            self.dialog_critical('Course Code not selected.')
            return False	
        if self.validate() == False:
            return
        code = 'Y' + self.syllYear + self.inst + '_' +self.codeLineEdit.text().strip()
        if code != self.courseCode:
            self.dialog_critical('Main Course Code, Syllabus Year and Institution type cannnot be modified.')
            return False	
        if 1:		
            sql = "update coursedetails set name = '%s', faculty = '%s', degree = '%s',  semester = '%s', texlanguage = '%s', qpmodel = '%s', duration = '%s', code2 = '%s', code3 = '%s', nmodules = '%s' where code = '%s';"%(self.courseName, self.faculty,self.degree, self.semester, self.language, self.qpModel, self.duration, self.code2, self.code3, self.moduleMarkString, self.courseCode)
            self.cursor.execute(sql)
            self.conn.commit()
        else:
            self.dialog_critical('Failed to Modify Course %s.'%code)
        self.updateCourseList()

        cpw1 = self.pass1LineEdit.text()
        cpw2 = self.pass2LineEdit.text()
        if cpw1 == '' and cpw2 == '':
            self.dialog_critical('Modified %s.'%self.courseCode)                
            return
        if len(cpw1) < 6:
            self.dialog_critical('Course password must have minimum 6 characters.')
            return False	
        if cpw1 != cpw2:
            self.dialog_critical('Course passwords not matching.')
            return False	
        
        sql = "alter user %s with password '%s'"%(code.lower(), cpw1)
        try:
            self.cursor.execute(sql)
            self.conn.commit()
        except:
            self.dialog_critical('Failed to Modify password of %s.'%code.lower())	
            return		
        self.msg.setText('Course Password Changed')
                
    def addCourse(self):	
        if self.validate() == False:	# check fields except the password
            return
        cpw = self.pass1LineEdit.text()
        if len(cpw) < 6:
            self.dialog_critical('Course password must have minimum 6 characters.')
            return False	
        if cpw != self.pass2LineEdit.text():
            self.dialog_critical('Course passwords not matching.')
            return False	
            
        code = 'Y' + self.syllYear + self.inst + '_' +self.codeLineEdit.text().strip().upper()
    
        try:
            sql = "CREATE TABLE %s(modnum int, unitnum varchar, qtype CHAR(1), qnum int, mark int, level VARCHAR, outcome varchar, author VARCHAR, qtext VARCHAR, atext VARCHAR, figname VARCHAR, figure bytea);\n"%code	
                        
            #print (sql)
            self.cursor.execute(sql)
            sql = "create user %s with password '%s';"%(code, cpw)
            self.cursor.execute(sql)
            sql = "grant all on table %s to %s;"%(code,code)
            self.cursor.execute(sql)
            sql = "grant select on table %s to reader;"%(code)
            self.cursor.execute(sql)
            
            sql = "insert into coursedetails (code, syllabusyear,inst, name, faculty, degree,  semester, texlanguage, qpmodel, duration,nmodules,code2,code3) values ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s','%s','%s','%s','%s');\n"%(code, self.syllYear, self.inst, self.courseName, self.faculty, self.degree, self.semester, self.language, self.qpModel, self.duration, self.moduleMarkString, self.code2, self.code3)	
            self.cursor.execute(sql)
        except:
            self.dialog_critical('Failed to add Course %s.'%code)
            self.cursor.execute('rollback;')
            return
        self.conn.commit()
        self.updateCourseList()
        self.courseCode = code
        self.dialog_critical('Added %s.'%code)
                
    def delCourse(self):
        code = 'Y' + self.syllYear + self.inst + '_' +self.codeLineEdit.text().strip()
        reply = QMessageBox.question(GUI, 'Delete Course', 'Proceed with Delete ?',QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.No:
            return	
        try:
            sql = "delete from coursedetails where code = '%s'"%code
            self.cursor.execute(sql)		
            sql = "drop table if exists %s;"%code
            self.cursor.execute(sql)		
            sql = "drop user if exists %s;"%code
            self.cursor.execute(sql)
            self.conn.commit()
        except:
            self.dialog_critical('Error: Delete Course %s'%code)
            self.cursor.execute('rollback;')			
        self.updateCourseList()
                
app = QApplication(sys.argv)   # Main program
GUI = MainWindow()
app.exec_()		# main loop

