#!/usr/bin/python3
'''
OpenQP : Question Bank Management System
Authors  : Ajith Kumar B.P (bpajith@gmail.com)
           T K Muraleedharan (muraleedharantk@kannuruniv.ac.in)
License : GNU GPL version 3
Last edit : 11-May-2025
'''

import psycopg2, subprocess, os, sys, random
import numpy as np
from random import randint 
from datetime import datetime, timedelta
from pathlib import Path
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
import subprocess, os, sys, pathlib, time, psycopg2
from psycopg2 import OperationalError

''' --------------------  Question paper models -----------------
E -> Equal mark Modules
U -> Unequal module marks

Z -> Calicut Univ
P -> Paragraph A type of Calicut Univ
Y -> Providence College
A -> Providence PG Botany, Commerce & Computer science
B -> Providence PG Physics
C -> Providence PG maths
X -> Kannur University
W -> Alternate 50 mark scheme of Kannur Univ.
'''
# CU
ZU70 = {'Ma':3,'Mb':6,'Mc':10,'Na':8,'Nb':6,'Nc':1,'Ta':10,'Tb':8,'Tc':2} #CU UG
ZU50 = {'Ma':2,'Mb':6,'Mc':10,'Na':8,'Nb':4,'Nc':1,'Ta':10,'Tb':5,'Tc':2} #CU UG
PU50 = {'Ma':20,'Mb':6,'Mc':10,'Na':1,'Nb':4,'Nc':1,'Ta':2,'Tb':5,'Tc':2} #paragraph type 50 

# Providence college
XU80 = {'Ma':4,'Mb':7,'Mc':13,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}  # err ??
YU70 = {'Ma':3,'Mb':6,'Mc':10,'Na':8,'Nb':6,'Nc':1,'Ta':10,'Tb':8,'Tc':2} #PC UG
YU50 = {'Ma':2,'Mb':6,'Mc':10,'Na':8,'Nb':4,'Nc':1,'Ta':10,'Tb':5,'Tc':2} #PC UG
AU30 = {'Ma':2,'Mb':3,'Mc':5,'Na':4,'Nb':4,'Nc':2,'Ta':7,'Tb':7,'Tc':4}
BU30 = {'Ma':1,'Mb':5,'Mc':3,'Na':8,'Nb':2,'Nc':4,'Ta':8,'Tb':4,'Tc':7} # PC physics
CU30 = {'Ma':1,'Mb':2,'Mc':5,'Na':8,'Nb':6,'Nc':2,'Ta':8,'Tb':9,'Tc':4} # PC maths

# KU
XE70 = {'Ma':3,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':2,'Ta':8,'Tb':6,'Tc':3,}
XE50 = {'Ma':2,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':1,'Ta':8,'Tb':6,'Tc':2} # KU colleges
WE50 = {'Ma':2,'Mb':6,'Mc':8,'Na':4,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':4} # KU department
        
XE15 = {'Ma':1,'Mb':2,'Mc':3,'Na':4,'Nb':4,'Nc':1,'Ta':6,'Tb':6,'Tc':2}
XE32 = {'Ma':1,'Mb':3,'Mc':6, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}
XE35 = {'Ma':2,'Mb':4,'Mc':7,'Na':6,'Nb':4,'Nc':1,'Ta':8,'Tb':6,'Tc':2}
XE40 = {'Ma':2,'Mb':3,'Mc':7, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}
XE48 = {'Ma':1,'Mb':5,'Mc':9, 'Na':6,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}
XE60 = {'Ma':3,'Mb':5,'Mc':10,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}
XE80 = {'Ma':4,'Mb':7,'Mc':13,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}

qpMap ={'ZU70':ZU70,'ZU50':ZU50,'PU50':PU50, 'YU70':YU70,'YU50':YU50,\
'AU30':AU30, 'BU30':BU30, 'CU30':CU30, 'XE70':XE70, 'XE50':XE50, 'WE50':WE50, 'XE35':XE35,\
'XE15':XE15, 'XE32':XE32, 'XE40':XE40, 'XE48':XE48, 'XE60':XE60, 'XE80':XE80, 'XU80':XU80}

            #modnum,unitnum,qtype,qnum,mark,level,outcome,author,qtext,atext
QMODNUM = 0 	# indices for the fields in Data Table
QUNITN = 1
QTYPE = 2
QSN = 3
QMARK = 4
QLEVEL = 5
QOUTC = 6
QAUTHOR = 7
QTEXT = 8
QATEXT = 9
QUSED = 10    	# added by QP generator
QPART2 = 11   	# second question of C
SINGLEQL = 11	# length of the list after adding the 'used' flag


pgTimeout = 20000
levelTypes = ['Remember','Understand','Apply', 'Analyze', 'Evaluate','Create','inactive']
#Display strings for Bloom's levels. Inactive questions marked as 'X'
BTL = {'Remember':'R1','Understand':'U2','Apply':'A3', 'Analyze':'A4', 'Evaluate':'E5','Create':'C6','inactive':'X'}

outcomeTypes = ['CO1','CO2','CO3','CO4','CO5','CO6','CO7']

CC1 = 0		# Indices of selected items from table 'coursedetails'
CC2 = 1
CC3 = 2
QPMODEL = 3
NMODS = 4
NAME = 5
LANG = 6

if sys.platform.startswith("win"):	# MS Windows
    pdfviewer="start"
    shell = True
else:
    pdfviewer="evince"
    shell = False	

#Easy way to store the marks of A,B & C types for different QP models
#mabc = {'M150A':10, 'M150B':15, 'M150C':25,'M50A':2, 'M50B':6, 'M50C':10,'M70A':3, 'M70B':6, 'M70C':10}

class Course:
    def __init__(self):
        self.myname = None
        self.myhost = None
        self.myport = None
        self.mydb = None
        self.courseCode = None
        self.conn = None
        self.cursor = None
        self.language = 'english'	# needed for generating LaTex
        # modNum, qType and qNumber uniquely defines the question
        self.modNum = None			
        self.qType = None
        self.qNumber = None
        self.qMark = None
        self.unitNum = None
        self.qLevel = None
        self.outcome = None
        self.numQ = 0		#number of existing questions, of the current Type
        self.courseList = []
        self.codeFilter = ''	
        self.editing = False
        self.connected = False
        
    def readConfig(self): # Get the database details from config.txt
        dialog = QFileDialog()
        path, _ = dialog.getOpenFileName(GUI, "Open Configuration file (config.txt)", "","Text Files(*.txt)")        
        if len(path) == 0:  # user cancelled selection
            return		
        
        try:
            f = open(path)
            while 1:
                ss = f.readline().strip()
                if ss == '':
                     break
                if ss[0] == '#':		#comments
                     continue
                item = ss.split(':')
                if item[0] == 'name':
                    self.myname = item[1]
                elif item[0] == 'host':
                    self.myhost = item[1]
                elif item[0] == 'port':
                    self.myport = int(item[1])
                elif item[0] == 'dbname':
                    self.mydb = item[1]
        except:
            GUI.dialog_critical('Config file not found')
            return			

    def makeConnection(self, user='dummy', pw='dum123'):
        print('Connecting to %s Server ...'%self.myname, end =' ')	
        sys.stdout.flush()		
        conn = psycopg2.connect(database=self.mydb, host=self.myhost, user=user, password=pw, port = self.myport)
        cursor = conn.cursor()
        print('Done. Reading course info..')	
        self.connected = True
        return conn,cursor
            
    def updateCourseList(self):
        if course.conn != None:
            msg = 'Editing a Course. Quit that and Connect Again ?'
            reply = QMessageBox.question(GUI, 'Connect to DB', msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
            if reply == QMessageBox.No:
                return
            self.conn.close()	
            self.conn = None
            self.cursor = None
            self.editing = False
            GUI.qeditor.setLineWrapMode(0)

        GUI.ccLineEdit.setText('')	
        self.codeFilter = ''	
        self.courseList = []
        if 1:
            conn,cursor = self.makeConnection()
            sql =  'select code,code2,code3,qpmodel,nmodules,name,texlanguage from coursedetails order by code;'
            cursor.execute(sql)
            res = cursor.fetchall()
            conn.close()
            print ('Fetched data')
        else:
            GUI.dialog_critical('Connection to Database failed.')
            return		
        for k in res:
            self.courseList.append(k)
        GUI.setWindowTitle('%s Server. %d Courses available.'%(self.myname,len(self.courseList)))
        self.displayCourses()

    def displayCourses(self):
        if self.courseList == []:
            return
        #print('In displayCourses')		
        txt = ''
        GUI.qeditor.setPlainText(txt)	
        if self.courseList[0][QPMODEL][1] == 'U': # unequal module marks
            unequal = True	
            txt = '%-22s %-12s %-12s %-4s %-14s %-50s\n'%('Course Code','Code2','Code3','QP','Module:Marks','Course Name')
        else:
            unequal = False	
            txt = '%-22s %-12s %-12s %-4s %-5s %-50s\n'%('Course Code','Code2','Code3','QP','nMods','Course Name')
            
        GUI.courseMenu.clear()
        count = 0
        for row in self.courseList:
            ss = row[CC1] + row[CC2] + row[CC3]
            if self.codeFilter != '' and self.codeFilter not in ss:  	
                continue		
            GUI.courseMenu.addAction(row[0], lambda item=row[0]: self.selectCourse(item))
            if unequal == True:	
                s = '%-22s %-12s %-12s %-4s %-14s %-50s\n'%(row[CC1], row[CC2], row[CC3], row[QPMODEL],row[NMODS], row[NAME].strip())
            else:
                s = '%-22s %-12s %-12s %-4s %-5s %-50s\n'%(row[CC1], row[CC2], row[CC3], row[QPMODEL],row[NMODS][0], row[NAME].strip())
                
            txt += s[:-1] + '\n'  
            count += 1			
        GUI.RCselectButton.setText('Select Course(%d)'%count)			
        GUI.qeditor.setPlainText(txt)	
        
    def selectCourse(self, code):  # select a Course from DB
        if self.conn != None:
            msg = 'deselect %s and select %s ?'%(self.courseCode,code)
            reply = QMessageBox.question(GUI, 'Select Course', msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
            if reply == QMessageBox.No:
                return			
            self.conn.close()
            self.conn = None
            self.cursor = None
            GUI.msg.setText('Disconnected from %s.'%self.courseCode)	
            
        pw = GUI.getPassword(code)	
        if pw == None:
            GUI.dialog_critical('No password obtained for %s.'%code)	
            return
        try:
            conn,cursor = self.makeConnection(code.lower(), pw)
            cursor.execute(f"SET statement_timeout = {pgTimeout}")				
            cursor.execute("SELECT count(*) FROM %s"%code)	
            res = cursor.fetchone()
        except OperationalError as e:
            GUI.dialog_critical('%s'%e)
            return
        self.conn = conn
        self.cursor = cursor	# to be used by  setQN, import and export
        self.courseCode = code
        self.user = code.lower()
        self.passwd = pw
        self.language = 'english'
        self.modNum = None		# modNum, qType and qNumber defines the question
        self.qType = None
        self.qNumber = None		
        for c in self.courseList:
            if c[0] == code:		# Store the required parameters only
                self.courseName = c[NAME]
                self.language = c[LANG]
                self.qpModel = c[QPMODEL]
                self.numModules = eval(c[NMODS][0])
            
        GUI.moduleNumberMenu.clear()
        for mn in range(1,self.numModules+1):
            s = '%d'%mn
            GUI.moduleNumberMenu.addAction(s, lambda item=s: self.setModuleNumber(item))
        GUI.moduleNumberButton.setMenu(GUI.moduleNumberMenu)
        mark = course.qpModel
        if mark[-1].isalpha():
            mark = course.qpModel[:-1]
        mark = mark[1:]
        s = '%s (%d Questions). %s marks paper'%(self.courseCode,res[0], mark)
        GUI.setWindowTitle(s)				
        self.clearQinputs()
        GUI.moduleNumberButton.setText('Module# ')
        GUI.questionTypeButton.setText('Q Type')
        GUI.qNumberButton.setText('Question# ')
        self.modNum = None
        self.qtype = None
        self.qNumber = None		
        self.editing = True
        GUI.qeditor.setLineWrapMode(1)
        GUI.msg.setText('Editing %s (%s). %d Questions'%(code,self.courseName,res[0]))	
        
        # End of selectRemoteCourse

    def clearQinputs(self):  
        GUI.qLevelTypeButton.setText('Level:'+levelTypes[0])
        GUI.outcomeTypeButton.setText('Outcome:'+outcomeTypes[0])
        GUI.qeditor.setPlainText('')				
        GUI.aeditor.setPlainText('')				
        
    def setQNmenu(self):	
        if self.courseCode == None or self.qType == None or self.modNum == None:
            GUI.dialog_critical('Course/Module/Type not selected.')
            return	
        sql = "SELECT count(*) FROM %s where qtype = '%s' and modNum = %s"\
            %(self.courseCode, self.qType, self.modNum)
        #print(sql)
        self.cursor.execute(sql)	
        self.numQ = self.cursor.fetchone()[0]
        GUI.qNumberMenu.clear()
        self.clearQinputs()	
        for qn in range(1, self.numQ+1):
            s = '%d'%qn
            GUI.qNumberMenu.addAction(s, lambda item=s: self.importQuestion(item))
        GUI.qNumberButton.setMenu(GUI.qNumberMenu)
        GUI.msg.setText('Selected Module %s, Type %s, found %d Questions'%(self.modNum,self.qType, self.numQ))	

    def setModuleNumber(self, n):
        if self.courseCode == None:
            GUI.dialog_critical('Course not selected.')
            return			
        self.modNum = int(n)
        if self.qType != None:
            self.setQNmenu()
        GUI.moduleNumberButton.setText('Module# %s'%n)
        GUI.qNumberButton.setText('Question# ')
        GUI.msg.setText('Selected Module %d'%(self.modNum))	

    def setMark(self, m):
        self.qMark = m
        GUI.markButton.setText('%s marks'%m)
        self.qMark = eval(m)
        GUI.msg.setText('Mark set to %s'%m)	

    def setQtype(self, qt):
        self.qType = qt
        if self.modNum != None:
            self.setQNmenu()
        GUI.questionTypeButton.setText('%s Type'%qt)
        GUI.qNumberButton.setText('Question# ')
        GUI.msg.setText('Question Type set to %s'%(qt))	

    def getFigures(self, tex):
        fragments = tex.split('\\includegraphics')
        nfig = len(fragments)
        for k in range(1,nfig):
            s1 = fragments[k]
            s2 = s1.split('{')[1]
            fig = s2.split('}')[0]
            if os.path.exists(fig):
                continue
            try:
                sql = "SELECT figname,figure FROM %s where modNum = %s and figname = '%s'"%(self.courseCode, self.modNum, fig)
                #print(sql)	
                self.cursor.execute(sql)	
                dbresults = self.cursor.fetchall()	# will take only one.
                #print(dbresults)
                if dbresults == []:
                    continue
                fname = dbresults[0][0]		
                figure = dbresults[0][1]
                f = open(fname,'wb')
                f.write(figure)
                f.close()
                GUI.msg.setText('Figures %s saved'%fname)
            except OperationalError as e:
                GUI.dialog_critical('Import figure: %s'%e)
                return
                
    def makeMarkMenu(self):
        index = 'M'+self.qType.lower()
        mc = qpMap[self.qpModel][index]
        
        if self.qType == 'C':    # for C type, half and full options
            allowedMarks = [mc] # [mc, mc//2]
        else:
            allowedMarks = [mc]	
        GUI.markMenu.clear()
        for mark in allowedMarks:	
            s = '%d'%mark
            GUI.markMenu.addAction(s, lambda item=s: self.setMark(item))

    def importQuestion(self,qn):	# Selecting a Question from the menu imports it
        if self.courseCode == None or self.qType == None or self.modNum == None:
            GUI.dialog_critical('Course/Module/Type not selected.')
            return	
        try:
            sql = "SELECT modnum, unitnum, qtype, qnum, mark, level, outcome, author, qtext, atext FROM %s where qtype = '%s' and modnum = %s and qnum = %s;"%(self.courseCode, self.qType, self.modNum, qn)
            self.cursor.execute(sql)	
            res = self.cursor.fetchall()  	# only one record expected	
            if res == []:
                GUI.dialog_critical('Error: Missing Question # : %s'%qn)
                return				
        except OperationalError as e:
            GUI.dialog_critical('Import Question: %s'%e)
            return
        res = res[0]
        self.qNumber = int(qn)
        GUI.qNumberButton.setText('Question# %d'%self.qNumber)
        self.getFigures(res[QTEXT] + res[QATEXT])
        GUI.qeditor.setPlainText(res[QTEXT])
        GUI.aeditor.setPlainText(res[QATEXT])
        GUI.authorEdit.setText(res[QAUTHOR].strip())
        self.makeMarkMenu()
        self.qMark = res[QMARK]
        GUI.markButton.setText('%d Marks'%self.qMark)		
        self.unitNum = res[QUNITN]
        GUI.unitLineEdit.setText('%s'%self.unitNum)        
        self.qLevel = res[QLEVEL]
        self.outcome = res[QOUTC]
        GUI.qLevelTypeButton.setText('Level:'+self.qLevel)
        GUI.outcomeTypeButton.setText('Outcome:'+self.outcome)
        GUI.msg.setText('Selected Question %s for editing'%(self.qNumber))
        self.updateTitle()

    def putFigures(self,tex):
        fragments = tex.split('\\includegraphics')	#search included figures
        figlist = []
        nfig = len(fragments)
        for k in range(1,nfig):
            s1 = fragments[k]
            s2 = s1.split('{')[1]
            fig = s2.split('}')[0]
            if os.path.exists(fig) == False:
                GUI.dialog_critical('File %s NOT found.'%fig)
                return				
            figlist.append(fig.strip())
        #print(figlist)	
        
        # search the present module for the same figure names, to avoid duplication
        for figname in figlist:
            sql = "SELECT figname FROM %s where modnum = %s and  figname = '%s'"%(self.courseCode, self.modNum, figname);
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
            if res != []:	# update the existing figure
                drawing = open(figname, 'rb').read()
                try:
                    img = psycopg2.Binary(drawing)
                    sql = "update %s set figure = %s where modnum = %d and figname ='%s';"%(self.courseCode, img, self.modNum,figname)
                    self.cursor.execute(sql)
                except OperationalError as e:
                    GUI.dialog_critical('%s'%e)
                    self.cursor.execute('rollback;')			
                    return
            else:	# insert new figure	
                try:
                    drawing = open(figname, 'rb').read()
                    img = psycopg2.Binary(drawing)
                    sql = "insert into %s(modnum,figname, figure) values (%d,'%s', %s);"%(self.courseCode, self.modNum,figname, img)
                    self.cursor.execute(sql)
                except OperationalError as e:
                    self.cursor.execute('rollback;')			
                    GUI.dialog_critical('%s'%e)
                    return			
        return True			
        
    def updateQuestion(self):
        if self.qNumber == None or self.qType == None or self.modNum == None:
            GUI.dialog_critical('No Question under Edit.')
            return	
        qt = GUI.qeditor.toPlainText().strip()
        at = GUI.aeditor.toPlainText().strip()	
        author = GUI.authorEdit.text().strip()	
        unit = GUI.unitLineEdit.text().strip()	
        if len(author) < 3:
            GUI.dialog_critical('Author name too short.')
            return				
        if len(qt) < 5:
            GUI.dialog_critical('Question too short.')
            return				
        self.splCharCheck(qt+at+author)				
                                                        
        qt = qt.replace("'","''")	#handle single quotes in SQL query		
        at = at.replace("'","''")			
        author = author.replace("'","''")		

        sql = "update %s SET qtext='%s', atext='%s', author = '%s', unitnum = '%s', mark = %s, level = '%s', outcome = '%s' where  modnum = %s and qtype = '%s' and qnum = %s;"%(self.courseCode, qt,at,author,unit, self.qMark, self.qLevel, self.outcome, self.modNum, self.qType, self.qNumber)
        try:
            self.cursor.execute(sql)	
        except OperationalError as e:
            self.cursor.execute('rollback;')			
            GUI.dialog_critical('%s'%e)
            return
            
        if self.putFigures(qt+at) != True:
            return
        self.conn.commit()
        GUI.dialog_critical('Written changes to the QBank Database')
        
    def addQuestion(self):
        if self.qType == None or self.modNum == None:
            GUI.dialog_critical('Select Module Number and Question Type.')
            return	
        self.setQNmenu()  # get the number of existing questions
        msg = 'Create a new %s Type Question (#%d) in Module %s'%(self.qType, self.numQ+1, self.modNum)
        reply = QMessageBox.question(GUI, 'New Question', msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.No:
            return

        index = 'M'+self.qType.lower()
        mark = qpMap[self.qpModel][index]
        print(mark)
        level = levelTypes[0]
        outcome = outcomeTypes[0]
        qtext = ''
        author = ''
        sql = "insert into %s (qtext,atext,author,modnum,qtype,qnum,unitnum,mark,level,outcome) values ('%s', '%s', '%s', %d, '%s', %d, '%s', %d, '%s','%s');"%(self.courseCode,qtext,'',author,self.modNum,self.qType, self.numQ+1,'0', mark, level,outcome)
        #print (sql)
        try:
            self.cursor.execute(sql)	
            self.qNumber = self.numQ + 1
            self.qMark = mark
            self.qLevel = level
            self.outcome = outcome
            self.setQNmenu()
            self.clearQinputs()
            self.makeMarkMenu()		
            GUI.markButton.setText('%d Marks'%self.qMark)				
            GUI.qNumberButton.setText('Question# %d'%self.qNumber)		
            self.updateTitle()
            GUI.msg.setText('Added New Question. Fill the values and Update.')
        except OperationalError as e:
            self.cursor.execute('rollback;')			
            GUI.dialog_critical('%s'%e)
        self.conn.commit()

    def setPassword(self):
        if self.cursor == None:
            GUI.dialog_critical('No Course Selected.')
            return			
        cpw = GUI.Pass1LineEdit.text()
        if len(cpw) < 6:
            GUI.dialog_critical('Must have minimum 6 characters.')
            return	
        if cpw != GUI.Pass2LineEdit.text():
            GUI.dialog_critical('Passwords not matching.')
            return
        sql = "alter role %s with password '%s';"%(self.courseCode,cpw)
        try:
            self.cursor.execute(sql)	
            self.conn.commit()		
            GUI.dialog_critical('Course Password changed.')
        except:
            GUI.dialog_critical('Course Password changing failed.')		

    def updateTitle(self):
        s = 'Editing %s (QP Model %s) | Module %d | QType %s | Question %d'%(self.courseCode, course.qpModel, self.modNum, self.qType, self.qNumber)
        GUI.setWindowTitle(s)
        
    def setqLevel(self, qlevel):
        GUI.qLevelTypeButton.setText('Level: '+qlevel)
        self.qLevel = qlevel
        
    def setOutcome(self, outcome):
        GUI.outcomeTypeButton.setText('Outcome: '+outcome)
        self.outcome = outcome
        
    def setCodeFilter(self):		
        if self.connected == False or self.editing == True:
            return
        self.codeFilter = GUI.ccLineEdit.text().upper()
        self.displayCourses()
        
    #------------ PDF generation functions ------------------			
    def splCharCheck(self, tex):
        spchars = '#@$&%^'
        for c in spchars:
            if c in tex:
                GUI.msg.setText('Warning: LaTex needs a backslash before %s, .'%c)
                
    def processTex(self, tex, fn):
        open(fn + '.tex', 'w', encoding="utf-16").write(tex)
        if os.path.exists(fn+'.aux'):
            os.unlink(fn+'.aux')
        if os.path.exists(fn+'.pdf'):
            os.unlink(fn+'.pdf')

        self.splCharCheck(tex)
        try:
            res = subprocess.run(["xelatex", '-halt-on-error', fn+'.tex'],capture_output=True,text=True)
            #print('ret = %d\n'%res.returncode)
            if res.returncode != 0:
                ss ='Error from "xelatex":\n%s'%res.stdout[-200:]
                GUI.dialog_critical(ss)
            else:
                ss ='Generated %s.pdf\n'%fn
                p2 = subprocess.Popen([pdfviewer, fn+'.pdf'],shell=shell)
        except:
                GUI.dialog_critical('Calling Latex compiler failed')
                return
            
    def texHeader(self):
        tex =  '\\documentclass[12pt, a4paper]{article}\n'
        tex += '\\usepackage{graphicx}\n'
        tex += '\\usepackage{amsmath}\n \\usepackage{amssymb}\n'
        tex += '\\usepackage[, headsep=0pt, left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'
        tex += '\\usepackage{enumitem}\n'
        tex += '\\usepackage{multicol}\n'
        tex += '\\usepackage{fancyhdr}\n'
        tex += '\\usepackage{fontspec}\n'
        tex += '\\usepackage{tfrupee}\n'
        
        langbeg = ''
        langend = ''
        tag = 'english'
        if self.language.lower() == 'malayalam':
            tag = 'malayalam'
            langbeg = '\\begin{malayalam}'
            langend = '\\end{malayalam}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{malayalam}\n'
            tex += '\\newfontfamily\\malayalamfont[Script=Malayalam]{Rachana}'
        elif self.language.lower() == 'tamil':
            tag = 'tamil'	
            langbeg = '\\begin{tamil}'
            langend = '\\end{tamil}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{tamil}\n'
            tex += '\\newfontfamily\\tamilfont[Script=Tamil]{Lohit-Tamil.ttf}\n'
        elif self.language.lower() == 'kannada':
            tag = 'kannada'	
            langbeg = '\\begin{kannada}'
            langend = '\\end{kannada}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{kannada}\n'
            tex += '\\newfontfamily\\kannadafont[Script=Kannada]{Lohit-Kannada.ttf}\n'
        elif self.language.lower() == 'hindi':
            tag = 'hindi'	
            langbeg = '\\begin{hindi}'
            langend = '\\end{hindi}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{hindi}\n'
            tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
        elif self.language.lower() == 'sanskrit':
            tag = 'hindi'	
            langbeg = '\\begin{hindi}'
            langend = '\\end{hindi}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{hindi}\n'
            tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
        elif self.language.lower() == 'arabic':
            tag = 'Arabic'
            langbeg = '\\begin{Arabic}'
            langend = '\\end{Arabic}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{arabic}\n'
            tex += '\\setmainfont[Script=Arabic,Scale=1.1]{Scheherazade}\n'	
        elif self.language.lower() == 'urdu':
            tag = 'urdu'
            langbeg = '\\begin{urdu}'
            langend = '\\end{urdu}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{urdu}\n'
            tex += '\\newfontfamily\\urdufont[Script=Arabic,Scale=1.0] {NafeesNastaleeq.ttf}\n'
        elif self.language.lower() == 'french':
            tag = 'english'
            langbeg = ''
            langend = ''	
            #tex += '\\usepackage[french]{babel}'
            #tex += '\\usepackage[autolanguage]{numprint}' 			

        tex += '\\pagestyle{fancy}\n'
        cname = '%s : Module %d'%(self.courseCode.replace('_','\\_'),self.modNum)
        pg = 'page'
        if tag != 'english':
            cname = '\\begin{english}%s\\end{english}'%cname
            pg = '\\begin{english}%s\\end{english}'%pg
        tex += '\\lhead{%s}\n'%cname
        tex += '\\chead{} \\rhead{%s\\thepage}\n'%pg
        tex += '\\lfoot{}\\cfoot{}\\rfoot{}\n'
        return tex,tag,langbeg,langend

    def singlePDF(self):
        if self.qNumber == None:
            return		
        tex,tag,langbeg,langend =  self.texHeader()			
        tex += '\\begin{document}\n\\noindent\n'
        tex += langbeg
        tex += GUI.qeditor.toPlainText().strip()
        at = GUI.aeditor.toPlainText().strip()
        if tag != 'english':
            ans = '\\begin{english}Hint:\\end{english}'+ at
            bts = '\\begin{english}%s:%s\\end{english}'%(BTL[self.qLevel],self.outcome)
        else:
            ans = 'Hint: '+ at
            bts = '%s:%s'%(BTL[self.qLevel],self.outcome)			
        tex += '\n\\hfill %s\n'%bts
        tex += '\\\\\n%s\n'%ans 
        tex += langend
        tex += '\\end{document}\n'
        self.getFigures(tex)			
        #self.getPassages(tex)		
        self.processTex(tex, 'qqq')

    def listModule(self):
        if self.courseCode == None or self.modNum == None:
            GUI.dialog_critical('Need to select Module Number.')
            return			
        tex,tag,langbeg,langend = self.texHeader()	
                
        tex += '\n\\begin{document} \\noindent\n'	
        for qtype in ['A','B','C']:
            sql = "SELECT mark,qtext,atext,unitnum,level,outcome FROM %s where modNum = '%s' and \
            qtype = '%s' and figname is null order by qnum;"%(self.courseCode, self.modNum, qtype)
            #print(sql)	
            try:
                self.cursor.execute(sql)
                res = self.cursor.fetchall()
            except OperationalError as e:
                GUI.dialog_critical('%s'%e)
                return			
            if len(res) == 0:
                continue
            index = 'M' + qtype.lower()
            ABmark = qpMap[self.qpModel][index]	
            tex += '\\hrule \\vspace{0.05in}\n'
            if qtype == 'C':
                tex += 'Section %s\n'%(qtype)
            else:
                tex += 'Section %s (%d marks each)\n'%(qtype,ABmark)				
            tex += '\\begin{enumerate}\n'
            tex += langbeg		# include the \begin{language} for non-english				
            for row in res:		# process the results from DB
                mark = row[0]		
                qt = row[1]
                at = row[2]
                unitnum = row[3]
                level = row[4]
                outcome = row[5]
                if tag != 'english':
                    uns = '\\begin{english}Unit  %d \\end{english}'%unitnum
                else:
                    uns = '(Unit %s)'%unitnum					
                if tag != 'english':
                    ms = '\\begin{english} (%d marks) \\end{english}'%mark
                else:
                    ms = '(%d marks)'%mark
                if qtype != 'C':
                    ms = ''
                if tag != 'english':
                    bts = '\\begin{english}%s:%s\\end{english}'%(BTL[level],outcome)
                else:
                    bts = '%s:%s'%(BTL[level],outcome)	
                tex += '\\item %s \\hspace*{0pt} \\hfill %s%s%s\n'%(qt, uns,bts,ms)
                if tag != 'english':
                    tex += '\\subitem \\begin{english}Hints:\\end{english} %s\n'%at
                else:
                    tex += '\\subitem Hints: %s\n'%at
                    
            tex += langend		
            tex += '\\end{enumerate}\n'			
        tex += '\\hrule \\end{document}'
        self.getFigures(tex)			
        self.processTex(tex,'Module%d'%self.modNum)
                            
#----------------------- End of Class Course ----------------

helpmsg = "                       How to use the Editor\n\
1. Press the button 'Connect to DB'. \
Available Course Codes will be listed.\n\
2. Type a couple of letters from your Course Code in the 'CC Filter' (che for chemistry).\n\
3. Select the Course to be edited using the Pulldown Menu 'Select Course'\n\
\n\
4. Select a Module Number and Question Type (A,B or C)\n\
5. Press 'Add New Question' or select an existing one from the pulldown menu.\n\
6. Add Question Text, Unit number and Author. Choose the Level and Course Outcome.\n\
7. If there are figures, equations, tables etc., verify using the 'View Question' button.\n\
8. Press 'Update to DB' to store the question to the Database.\n\
      Repeat from step 5 until the selected Question Type is over.\n\
      Repeat the process for Other Question types\n\
      Press 'List Current Module' to generate a PDF output\n\
"

class MainWindow(QMainWindow):   # Creating main window class
    qpt = None		# thread for QP generation
    qptStartTime = None
    
    def dialog_critical(self, s):
        dlg = QMessageBox(self)
        dlg.setText(s)
        dlg.setIcon(QMessageBox.Critical)
        dlg.show()
    
    def getPassword(self, cname):
        dialog = QInputDialog(self)
        dialog.setFixedSize(700, 100)
        text, ok = dialog.getText(None, "Course %s"%cname, "Password?", 
        QLineEdit.Password)
        if ok and text:
            return text		
                
    def closeEvent(self, event):
        if course.cursor != None:
            course.cursor.close()
            print('closed cursor')
        if course.conn != None:
            course.conn.close()
            print('closed connection')
        
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        
        palette = QPalette()
        palette.setColor(QPalette.ColorRole(10), QColor(150,175,150)) 
        self.setPalette(palette)	
        
        self.setGeometry(100, 100, 1100, 800)			
        fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
        fixedfont.setPointSize(14)
        
        self.aeditor = QPlainTextEdit()
        self.aeditor.setFont(fixedfont)
        self.aeditor.setLineWrapMode(0)
        
        self.qeditor = QPlainTextEdit(helpmsg)
        self.qeditor.setFont(fixedfont)
        self.qeditor.setLineWrapMode(0)

        layout = QVBoxLayout()
        hbox1 = QHBoxLayout()
        hbox2 = QHBoxLayout()
        hbox3 = QHBoxLayout()
        
        self.dbSelectButton = QPushButton('Select Database')
        hbox1.addWidget(self.dbSelectButton)
        self.dbSelectButton.clicked.connect(course.readConfig)	
        
        self.dbConnectButton = QPushButton('Connect to DB')
        s = 'Connects to the Database and displays the list of the available courses\nRestrict the number of courses listed by typing a couple of letters from your course code in the Filter.\nThen select a course for editing using the "Select Course" pull down menu'
        self.dbConnectButton.setToolTip(s)
        hbox1.addWidget(self.dbConnectButton)
        self.dbConnectButton.clicked.connect(course.updateCourseList)	

        hbox1.addWidget(QLabel('CC Filter->'))	
        self.ccLineEdit = QLineEdit('')		
        hbox1.addWidget(self.ccLineEdit)
        self.ccLineEdit.setMaximumWidth(150)
        self.ccLineEdit.textChanged.connect(course.setCodeFilter)		
                        
        self.courseMenu = QMenu()
        self.RCselectButton = QPushButton('Select Course')
        s = 'Choose a Course for editing.\nPassword required for accessing a course'
        self.RCselectButton.setToolTip(s)
        self.RCselectButton.setMenu(self.courseMenu)
        hbox1.addWidget(self.RCselectButton)
        hbox1.addStretch()
        
        self.Pass1LineEdit = QLineEdit('')
        self.Pass1LineEdit.setMaximumWidth(100)
        self.Pass1LineEdit.setEchoMode(QLineEdit.Password)
        hbox1.addWidget(self.Pass1LineEdit)
        self.Pass2LineEdit = QLineEdit('')
        self.Pass2LineEdit.setMaximumWidth(100)
        self.Pass2LineEdit.setEchoMode(QLineEdit.Password)
        hbox1.addWidget(self.Pass2LineEdit)
        self.PassButton = QPushButton('Change password')
        s = 'Change the password for accessing this course'
        self.PassButton.setToolTip(s)
        self.PassButton.clicked.connect(course.setPassword)
        hbox1.addWidget(self.PassButton)		
        layout.addLayout(hbox1)
        
        self.moduleNumberButton = QPushButton('Module #')
        s = 'Select a Module.'
        self.moduleNumberButton.setToolTip(s)		
        hbox2.addWidget(self.moduleNumberButton)
        self.moduleNumberMenu = QMenu()
        self.moduleNumberButton.setMenu(self.moduleNumberMenu)
        self.questionTypeButton = QPushButton('Q Type')
        s = 'Select a Question Type.'
        self.questionTypeButton.setToolTip(s)		
        hbox2.addWidget(self.questionTypeButton)
        menu = QMenu()
        for qt in ['A', 'B', 'C']:
            menu.addAction(qt, lambda item=qt: course.setQtype(item))
        self.questionTypeButton.setMenu(menu)
        self.qNumberButton = QPushButton('Question #')
        s = 'Select a Question for editing.'
        self.qNumberButton.setToolTip(s)		
        hbox2.addWidget(self.qNumberButton)
        self.qNumberMenu = QMenu()
        self.qNumberButton.setMenu(self.qNumberMenu)	
        self.addButton = QPushButton('Add New Question')
        s = 'Add a New Question.'
        self.addButton.setToolTip(s)		
        self.addButton.clicked.connect(course.addQuestion)
        hbox2.addWidget(self.addButton)		
        hbox2.addStretch()
        
        self.makePDFButton = QPushButton('View Question')
        s = 'Show current Question in PDF format'
        self.makePDFButton.setToolTip(s)		
        self.makePDFButton.clicked.connect(course.singlePDF)
        hbox2.addWidget(self.makePDFButton)
        
        self.listButton = QPushButton('List Current Module')
        s = 'Displays all the questions in the selected Module\nin PDFformat.'
        self.listButton.setToolTip(s)		
        self.listButton.clicked.connect(course.listModule)
        hbox2.addWidget(self.listButton)
        
        layout.addLayout(hbox2)

        hbox0 = QHBoxLayout()
        hbox0.addWidget(QLabel(''))
        layout.addLayout(hbox0)

        self.markMenu = QMenu()
        self.markButton = QPushButton('Mark')
        hbox3.addWidget(self.markButton)
        self.markButton.setMenu(self.markMenu)

        self.authorEdit = QLineEdit('   ')
        self.authorEdit.setMaximumWidth(150)
        hbox3.addWidget(QLabel('Author'))
        hbox3.addWidget(self.authorEdit)

        hbox3.addWidget(QLabel('Unit->'))	
        self.unitLineEdit = QLineEdit('')		
        hbox3.addWidget(self.unitLineEdit)
        self.unitLineEdit.setMaximumWidth(50)

        course.qLevel = levelTypes[0]
        self.qLevelTypeButton = QPushButton('Level: '+ levelTypes[0])
        hbox3.addWidget(self.qLevelTypeButton)	
        menu = QMenu()
        for qs in levelTypes:
            menu.addAction(qs, lambda item=qs: course.setqLevel(item))
        self.qLevelTypeButton.setMenu(menu)
        
        course.outcome = outcomeTypes[0]
        self.outcomeTypeButton = QPushButton('Outcome: '+outcomeTypes[0])
        hbox3.addWidget(self.outcomeTypeButton)
        menu = QMenu()
        for qs in outcomeTypes:
            menu.addAction(qs, lambda item=qs: course.setOutcome(item))
        self.outcomeTypeButton.setMenu(menu)	
        hbox3.addStretch()
        self.updateButton = QPushButton('Update to DB')
        s = 'Write the question to the Database.'
        self.updateButton.setToolTip(s)		
        self.updateButton.clicked.connect(course.updateQuestion)
        hbox3.addWidget(self.updateButton)
        layout.addLayout(hbox3)

        split = QSplitter(Qt.Vertical)
        split.addWidget(self.qeditor)
        split.addWidget(self.aeditor)
        layout.addWidget(split)

        hbox5 = QHBoxLayout()
        layout.addLayout(hbox5)		
        self.msg = QLabel('msg')
        self.msg.setStyleSheet("background-color: white;")		
        hbox5.addWidget(self.msg)		

        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)  	# making it central widget
        self.setWindowTitle('Question Bank management: Client side editor program')
        self.show()
        if not os.path.exists('output'):
            os.mkdir('output')
        os.chdir('output')           

app = QApplication(sys.argv)   # Main program
course = Course()
GUI = MainWindow()
app.exec()		# main loop

