#!/usr/bin/python3
'''
OpenQP : Question Bank Management System
Authors  : Ajith Kumar B.P (bpajith@gmail.com)
           T K Muraleedharan (muraleedharantk@kannuruniv.ac.in)
License : GNU GPL version 3
Last edit : 25-Mar-2025
'''

import psycopg2, subprocess, os, sys, random
from psycopg2 import OperationalError
import numpy as np
from random import randint 
from datetime import datetime, timedelta
from pathlib import Path
    
'''
E -> Equal mark Modules
U -> Unequal module marks

Z -> Calicut Univ
P -> Paragraph A type of Calicut Univ
Y -> Providence College
A -> Providence PG Botany, Commerce & Computer science
B -> Providence PG Physics
C -> Providence PG maths
X -> Kannur University
W -> Alternate 50 mark scheme of Kannur Univ.
'''
# CU
ZU70 = {'Ma':3,'Mb':6,'Mc':10,'Na':8,'Nb':6,'Nc':1,'Ta':10,'Tb':8,'Tc':2} #CU UG
ZU50 = {'Ma':2,'Mb':6,'Mc':10,'Na':8,'Nb':4,'Nc':1,'Ta':10,'Tb':5,'Tc':2} #CU UG
PU50 = {'Ma':20,'Mb':6,'Mc':10,'Na':1,'Nb':4,'Nc':1,'Ta':2,'Tb':5,'Tc':2} #paragraph type 50 

# Providence college
XU80 = {'Ma':4,'Mb':7,'Mc':13,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}  # err ??
YU70 = {'Ma':3,'Mb':6,'Mc':10,'Na':8,'Nb':6,'Nc':1,'Ta':10,'Tb':8,'Tc':2} #PC UG
YU50 = {'Ma':2,'Mb':6,'Mc':10,'Na':8,'Nb':4,'Nc':1,'Ta':10,'Tb':5,'Tc':2} #PC UG
AU30 = {'Ma':2,'Mb':3,'Mc':5,'Na':4,'Nb':4,'Nc':2,'Ta':7,'Tb':7,'Tc':4}
BU30 = {'Ma':1,'Mb':5,'Mc':3,'Na':8,'Nb':2,'Nc':4,'Ta':8,'Tb':4,'Tc':7} # PC physics
CU30 = {'Ma':1,'Mb':2,'Mc':5,'Na':8,'Nb':6,'Nc':2,'Ta':8,'Tb':9,'Tc':4} # PC maths

# KU
XE70 = {'Ma':3,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':2,'Ta':8,'Tb':6,'Tc':3,}
XE50 = {'Ma':2,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':1,'Ta':8,'Tb':6,'Tc':2} # KU colleges
WE50 = {'Ma':2,'Mb':6,'Mc':8,'Na':4,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':4} # KU department
        
XE15 = {'Ma':1,'Mb':2,'Mc':3,'Na':4,'Nb':4,'Nc':1,'Ta':6,'Tb':6,'Tc':2}
XE32 = {'Ma':1,'Mb':3,'Mc':6, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}
XE35 = {'Ma':2,'Mb':4,'Mc':7,'Na':6,'Nb':4,'Nc':1,'Ta':8,'Tb':6,'Tc':2}
XE40 = {'Ma':2,'Mb':3,'Mc':7, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}
XE48 = {'Ma':1,'Mb':5,'Mc':9, 'Na':6,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}
XE60 = {'Ma':3,'Mb':5,'Mc':10,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}
XE80 = {'Ma':4,'Mb':7,'Mc':13,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5}

qpMap ={'ZU70':ZU70,'ZU50':ZU50,'PU50':PU50, 'YU70':YU70,'YU50':YU50,\
'AU30':AU30, 'BU30':BU30, 'CU30':CU30, 'XE70':XE70, 'XE50':XE50, 'WE50':WE50, 'XE35':XE35,\
'XE15':XE15, 'XE32':XE32, 'XE40':XE40, 'XE48':XE48, 'XE60':XE60, 'XE80':XE80, 'XU80':XU80}

semList = ['First','Second','Third','Fourth','Fifth','Sixth','Seventh', 'Eighth']

# Index of items selected from the course table 
QMODNUM = 0
QUNITN = 1
QTYPE = 2
QSN = 3
QMARK = 4
QLEVEL = 5
QOUT = 6
QTEXT = 7
QATEXT = 8
QUSED = 9       # added after fetching data
QPART2 = 10     # second question of C
SINGLEQL = 10	# length of the list after adding the 'used' flag
    
class Qpgen():
    def __init__(self, host, port, db, user, pw, year,month,code,ccdisp,sup):
        self.examYear = year
        self.examMonth = month
        self.qpCode = code
        self.ccdisp = ccdisp
        self.supple = sup
        self.host = host
        self.db = db
        self.port = port
        self.user = user
        self.passwd = pw
        self.conn = None
        self.cursor = None
        
        # Arrays to store the number of questions in each Part
        self.naq = None		
        self.nbq = None
        self.ncq = None
        # 2D Arrays to store the questions from each module
        self.Atypes = None
        self.Btypes = None
        self.Ctypes = None
        #Lists to store the selected questions
        self.AQ = []
        self.BQ = []
        self.CQ = []	
        self.moduleMarks = [None]*8
        self.scaledMarks = [None]*8
        self.logfile = None
        
    def log(self,s):
        self.logfile.write(s)
        self.logfile.flush()
    
    def fillMarks(self):
        print(self.qpModel)
        self.totalMarks = eval(self.qpModel[-2:])
        s = self.mmString.split(':')
        if self.qpModel[1] == 'E': # Equal marks to all Modules
            for k in range(self.numModules):
                self.moduleMarks[k] = self.totalMarks/self.numModules
        else:
            for k in range(self.numModules):
                self.moduleMarks[k] = eval(s[k+1])			
                         
        qpd = qpMap[self.qpModel]
        Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
        Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
        Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
        ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
        self.scaledTotal = 0
        for k in range(self.numModules):
            x = self.moduleMarks[k]*ratio
            self.scaledMarks[k] = x
            self.scaledTotal += x					
        self.Ma = Ma		# Used while filling the Question paper
        self.Mb = Mb
        self.Mc = Mc
        self.Na = Na
        self.Nb = Nb
        self.Nc = Nc
        self.Ta = Ta
        self.Tb = Tb
        self.Tc = Tc	
        
    def findCourse(self, courseCode):
        '''
        Fetches the "coursedetails" Table from Postgres and searches for the
        specified course.
        '''
        fn = '%s.txt'%courseCode
        print('opening ', fn)
        self.logfile = open(fn,'w')
        print('Searching for %s ...'%courseCode, end =' ')	
        sys.stdout.flush()		
        conn = psycopg2.connect(database = self.db, host=self.host, port=self.port, user='dummy' , password = 'dum123')
        cursor = conn.cursor()
        sql = "select code,code2,code3,name,semester,texlanguage,qpmodel,nmodules,duration,degree from coursedetails where code = '%s';"%courseCode
        cursor.execute(sql)
        res = cursor.fetchall()
        print('Done.')	
        sys.stdout.flush()		
        
        CC1 = 0; CC2 = 1; CC3 = 2; NAME = 3;  SEM = 4; LANG = 5; QPM = 6
        NMODS = 7; DUR = 8; DEG = 9
        for row in res:
            if row[0] == courseCode:
                self.courseCode = row[CC1]
                self.code2 = row[CC2]
                self.code3e = row[CC3]
                self.syllabusYear = self.courseCode[1:5]
                self.courseName = row[NAME]
                self.semester = row[SEM]
                self.language = row[LANG]
                self.qpModel = row[QPM]
                self.duration = row[DUR]
                self.degree = row[DEG]
                self.mmString = row[NMODS]	# colon separated fields. First is numModules
                self.numModules = eval(self.mmString.split(':')[0])
                self.fillMarks()
                self.log('%s: %s\nTypesetting in %s. '%(row[CC1],row[NAME], row[LANG]))
                self.log('Question Paper Type:%s. '%self.qpModel)
                #print('Question Paper Type:%s. '%self.qpModel)
                self.log('%d Modules. Marks %s\n'%(self.numModules,row[NMODS]))
                return True		
        return False

    def getFigures(self, tex):
        fragments = tex.split('\\includegraphics')
        nfig = len(fragments)
        for k in range(1,nfig):
            s1 = fragments[k]
            s2 = s1.split('{')[1]
            fname = s2.split('}')[0]
            try:
                sql = "SELECT figname,figure FROM %s where figname = '%s'"%(self.courseCode, fname)
                self.cursor.execute(sql)	
                dbresults = self.cursor.fetchall()	# will take only one.
                #print(dbresults)
                if dbresults == []:
                    continue
                fname = dbresults[0][0]		
                figure = dbresults[0][1]
                f = open(fname,'wb')
                f.write(figure)
                f.close()
                print('Figures %s saved'%fname)
            except OperationalError as e:
                GUI.dialog_critical('Import figure: %s'%e)
                return
      
    def fetchData(self):
        '''
        Fetches the question records of the specified course from the Postgres RDBMS and stores them
        to 2D(numModules x Number of questions) arrays Atypes, Ntypes and Ctypes.
        Stores the number of questions info in arrays naq, nbq and ncq.
        '''
        print('Fetching course data ...', end =' ')	
        sys.stdout.flush()	
        
        if 1:
            self.conn = psycopg2.connect(database=self.db, host=self.host, user=self.user, password = self.passwd,port=self.port)
            self.cursor = self.conn.cursor()	
            sql = "SELECT modnum, unitnum, qtype, qnum, mark, level, outcome, qtext, atext FROM %s where figname is null;"%self.courseCode
            self.cursor.execute(sql)	
            results = self.cursor.fetchall()
        else: #xcept OperationalError as e:
            self.log('%s: %s<br>'%(self.courseCode,e))
            return False
        print('Done.',len(results))	
        sys.stdout.flush()	

        # Count the number of questions in each section, type wise
        self.naq = np.zeros(self.numModules, dtype=int)
        self.nbq = np.zeros(self.numModules, dtype=int)
        self.ncq = np.zeros(self.numModules, dtype=int)

        for row in results:
            for k in range(self.numModules):	# k goes from 0 to self.numModules-1.
                if row[QTYPE] == 'A' and row[QMODNUM] == k+1:
                    self.naq[k] += 1
                if row[QTYPE] == 'B' and row[QMODNUM] == k+1:
                    self.nbq[k] += 1
                if row[QTYPE] == 'C' and row[QMODNUM] == k+1:
                    self.ncq[k] += 1
        
        self.log('Question distribution info:\nA types:%s.\nB types:%s.\nC types:%s\n'%(self.naq, self.nbq, self.ncq))
        # Make 3 array of objects, size self.numModules, to transfer the SQL query results
        self.Atypes = np.empty(self.numModules,object)
        self.Btypes = np.empty(self.numModules,object)
        self.Ctypes = np.empty(self.numModules,object)
                
        for k in range(self.numModules):	# Array size = self.numModules x NoQ in each (variable)
            self.Atypes[k] = np.empty(self.naq[k], object)
            self.Btypes[k] = np.empty(self.nbq[k], object)
            self.Ctypes[k] = np.empty(self.ncq[k], object)
        
        for k in range(len(results)):	# Copy to 3 2D Array of objects
            row = list(results[k])
            row.append(False)	# Add the 'QUSED' flag, set False			
            if row[QLEVEL] == 'inactive':
                row[QUSED] = True
            if row[QTYPE] == 'A':
                sec = row[QMODNUM]
                qn = row[QSN]
                self.Atypes[sec-1][qn-1] = row	# Indices from 0 to self.numModules-1
            if row[QTYPE] == 'B':
                sec = row[QMODNUM]
                qn = row[QSN]
                self.Btypes[sec-1][qn-1] = row
            if row[QTYPE] == 'C':
                sec = row[QMODNUM]
                qn = row[QSN]
                self.Ctypes[sec-1][qn-1] = row
            '''    
            if row[QFIGNAME] != None:	# Save if there is a figure
                f = open(row[QFIGNAME],'wb')
                f.write(row[QFIG])
                f.close()
            '''    	
        self.log('Total %d Questions fetched\n\n'%len(results))
        return True	
        
    #--------------- Select Questions for QP ---------------------
    def showDist(self, ia,ib,ic):
        t = self.Ta*self.Ma +self.Tb*self.Mb +self.Tc*self.Mc
        #print(self.moduleMarks, self.pm2, )
        #pat = '\nReport:\n'
        pat = 'Has %d Modules. Marks ['%(self.numModules)
        tm = 0
        for k in range(self.numModules):
            pat += '%3.0f '%self.moduleMarks[k]
            tm += self.moduleMarks[k]
        pat += '] = %d\n'%tm
        pat += 'All Questions %dx%d + %dx%d + %dx%d = %d marks.\n'\
        %(self.Ta,self.Ma,self.Tb,self.Mb,self.Tc,self.Mc,t)
        pat += 'Scaling gives the module wise marks as ['
        for k in range(self.numModules):
            pat += '%3.0f '%self.scaledMarks[k]
        pat += ']\n\n'
        pat += 'Decided to select Questions as shown below:\n'
         
        mtotal = 0
        mark = ''
        for k in range(self.numModules):
            mp = 0
            ss = ''
            pat += 'from Module %d | '%(k+1)
            if k in ic:
                nc = ic.count(k)
                mp += nc * self.Mc
                ss += '%s'%('C'*nc)
            if k in ib:
                nb = ib.count(k)
                mp += nb * self.Mb
                ss += '%s'%('B'*nb)
            if k in ia:
                na = ia.count(k)
                mp += na * self.Ma
                ss += '%s'%('A'*na)
            mtotal += mp
            pat += '%-7s | %d marks\n'%(ss,mp)
            mtotal += mp
            mark += '%d:'%mp
        self.log(pat+'\n')	
        
    #-------------different selection models----------------------	

    def selectAU30(self):   # Providence Bot., Com, and CS
        ia = []	
        ib = []
        ic = []			
        trial = 0
        for k in range(self.numModules):
            if  len(ic) < self.Tc:
                ic.append(k)
                ia.append(k)
        while len(ic) < self.Tc and trial < 30:	# fill C types, add an A also to each 
            n = randint(0,self.numModules-1)  
            trial += 1
            ic.append(n)
            ia.append(n)
                
        for k in range(self.numModules):	# fill A and B one round(exclude C filled)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            #if self.Mb+mark <= self.scaledMarks[k] and len(ib) < self.Tb:
            if len(ib) < self.Tb:               
                ib.append(k)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if  len(ia) < self.Ta:   
                ia.append(k)
            
        # fill remaining as per the moduleMarks values
        for n in range(50):		
            k = randint(0,self.numModules-1)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if len(ib) < self.Tb:
                ib.append(k)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if(self.Ma+mark <= self.scaledMarks[k]*1.1 and len(ia) < self.Ta):
                ia.append(k)
            if len(ia) == self.Ta and len(ib) == self.Tb and len(ic) == self.Tc:
                break
            
        if len(ia) != self.Ta or len(ib) != self.Tb or len(ic) != self.Tc:
            self.log('Failed to find questions for all %d sections '%self.numModules)
            print('Could not get questions from all sections ',ic,ib,ia)
            self.showDist(ia, ib, ic)		
            return False
        print(ia,ib,ic)
        return ia,ib,ic

    def selectBU30(self):   # Providence Physics, essay is B type
        ia = []	
        ib = []
        ic = []			
        trial = 0
        while len(ib) < self.Tb and trial < 30:	# fill B types, add an A also 
            n = randint(0,self.numModules-1)  
            trial += 1
            if n not in ib and self.nbq[n] != 0:  # All modules should have B type
                ib.append(n)
                ia.append(n)
            
        # fill remaining as per the moduleMarks values
        for n in range(50):		
            k = randint(0,self.numModules-1)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if(self.Mc+mark <= self.scaledMarks[k]*1.2 and len(ic) < self.Tc):
                ic.append(k)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if(self.Ma+mark <= self.scaledMarks[k]*1.1 and len(ia) < self.Ta):
                ia.append(k)
            if len(ia) == self.Ta and len(ib) == self.Tb and len(ic) == self.Tc:
                break
            
        if len(ia) != self.Ta or len(ib) != self.Tb or len(ic) != self.Tc:
            self.log('Failed to find questions for all %d sections '%self.numModules)
            print('Could not get questions from all sections ',ic,ib,ia)
            self.showDist(ia, ib, ic)		
            return False
        print(ia,ib,ic)
        return ia,ib,ic

    def selectCU30(self):   # Providence Maths
        ia = []	
        ib = []
        ic = []			
        trial = 0
        for k in range(self.numModules):
            ic.append(k)
        while len(ic) < self.Tc and trial < 10:
            n = randint(0,self.numModules-1)  
            trial += 1
            if self.ncq[n] != 0:  
                ic.append(n)

        for k in range(3):	# fill 3 Bs from each module
            ib.append(k)
            ib.append(k)
            ib.append(k)

        for n in range(10):		
            k = n%self.numModules #randint(0,self.numModules-1)
            if len(ia) < self.Ta:
                ia.append(k)
            if len(ia) == self.Ta and len(ib) == self.Tb and len(ic) == self.Tc:
                break
                        
        if len(ia) != self.Ta or len(ib) != self.Tb or len(ic) != self.Tc:
            self.log('Failed to find questions for all %d sections '%self.numModules)
            print('Could not get questions from all sections ',ic,ib,ia)
            self.showDist(ia, ib, ic)		
            return False
        print(ia,ib,ic)
        return ia,ib,ic

    def selectZU70(self):   # Calicut Univ 70 marks
        ia = []	
        ib = []
        ic = []			
        trial = 0
        while len(ic) < self.Tc and trial < 10:	# fill C types, add an A also to each 
            n = randint(0,self.numModules-1)  
            trial += 1
            if n not in ic and self.ncq[n] != 0:  # At least 2 modules should have C type
                ic.append(n)
                ia.append(n)

        for n in range(self.numModules * 2):	# fill B 2 rounds(exclude C filled)
            if len(ib) == self.Tb:
                break
            k = n % self.numModules
            mark = self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if self.Mb+mark <= self.scaledMarks[k] and len(ib) < self.Tb:   
                ib.append(k)
            
        # fill remaining as per the moduleMarks values
        for n in range(50):		
            k = n%self.numModules #randint(0,self.numModules-1)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if(self.Mb+mark <= self.scaledMarks[k]*1.1 and len(ib) < self.Tb and k not in ic):
                ib.append(k)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if(self.Ma+mark <= self.scaledMarks[k]*1.1 and len(ia) < self.Ta):
                ia.append(k)
            if len(ia) == self.Ta and len(ib) == self.Tb and len(ic) == self.Tc:
                break
            
        if len(ia) != self.Ta or len(ib) != self.Tb or len(ic) != self.Tc:
            self.log('Failed to find questions for all %d sections '%self.numModules)
            print('Err ',ic,ib,ia)
            self.showDist(ia, ib, ic)		
            return False
        print(ia,ib,ic)
        return ia,ib,ic

    def selectZU50(self):   # Calicut Univ 50 marks
        trial = 0
        ic = []			# fill C types, handle sections without C also
        while len(ic) < self.Tc and trial < 10:			
            n = randint(0,self.numModules-1)  
            trial += 1
            if n not in ic and self.ncq[n] != 0:
                ic.append(n)
                
        ib = []
        for n in range(self.numModules):	# fill B 1 round(exclude C filled)
            k = n % self.numModules
            mark = self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if self.Mb+mark <= self.scaledMarks[k] and len(ib) < self.Tb:   
                ib.append(k)
            
        ia = []	
        # fill remaining as per the moduleMarks values
        for n in range(50):		
            k = n%self.numModules #randint(0,nmods-1)
            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if(self.Mb+mark <= self.scaledMarks[k]*1.1 and len(ib) < self.Tb and k not in ic):
                ib.append(k)

            mark = self.Ma * ia.count(k) + self.Mb * ib.count(k) + self.Mc * ic.count(k)
            if(self.Ma+mark <= self.scaledMarks[k]*1.1 and len(ia) < self.Ta):
                ia.append(k)

            if len(ia) == self.Ta and len(ib) == self.Tb and len(ic) == self.Tc:
                break
            
        if len(ia) != self.Ta or len(ib) != self.Tb or len(ic) != self.Tc:
            self.log('Failed to find questions for all %d sections '%self.numModules)
            return False
        return ia,ib,ic
 
    # Kannur University QP models
    def selectWE50(self):  # rigid nmods = 8, KU Colleges 50 marks,14-Nov-2024	
        nmods = 8
        ic = []
        ib = []
        ia = []
        
        while 1:
            if len(ic) >= self.Tc:
                break
            n = randint(0, nmods//2 -1)*2   # 0, 2, 4 or 6			
            if n not in ic:
                ic.append(n)			# Add 2 more C types
                ic.append(n)			
                ia.append(n)			# and one A type  			
                            
        Csel = [ic[0], ic[0]+1, ic[2], ic[2]+1]
        
        for k in range(nmods):		# Add 4 B and A each
            if k not in Csel: 		
                ib.append(k)
                ia.append(k)

        while 1:		
            n = randint(0, nmods-1)	
            if len(ib) >= self.Tb:
                break
            if n not in Csel: 
                ib.append(n)

        print (ic,ib,ia)
        return ia,ib,ic	
    
    def selectXE70(self):	# Rigid logic, assuming 8 modules
        nmods = 8
        ic = []
        n = randint(0, nmods//2 -1)*2   # 0, 2, 4 or 6
        ic.append(n)		# Two C types from the same 
        ic.append(n)	
        while 1:
            n = randint(0, nmods//2 -1)*2   # 0, 2, 4 or 6			
            if n not in ic:
                ic.append(n)			# Add the third C type
                break

        # Mark the odd and even modules selected for C
        Csel = [ic[0], ic[0]+1, ic[2], ic[2]+ 1]  

        # B only from half, excluding C fully, 
        # A from noC half, also two from C3 top

        ib = []
        for k in range(nmods):		# Add Bs, excluding the C blocks
            if k not in Csel: 
                ib.append(k)

        for n in [0,2,4,6]:
            if len(ib) >= self.Tb:
                break
            if (n not in Csel) and ib.count(n) < 2: 
                ib.append(n + randint(0,1))
                
        ia = [ic[2], ic[2]]   		 	# Two from the third C type module
        for k in range(nmods):			# One each from the no C blocks
            if len(ia) >= self.Ta:
                break
            if k not in Csel:
                ia.append(k)
        while 1:		
            if len(ia) >= self.Ta:
                break
            n = randint(0, nmods-1)	
            if n not in Csel and (ia.count(n) + ib.count(n)) < 3: 
                ia.append(n)
                                
        #print (ic,ib,ia)		
        return ia,ib,ic

    def selectXE60(self): 		
        nmods = 8
        ic = []
        ib = []
        ia = []
        for k in range(0,nmods,2):
            ic.append(k)		# One C & A type from even ones 
            ia.append(k)	
        n = randint(0, nmods//2 -1)*2   # 0, 2, 4 or 6
        ic.append(n+1)		
        ia.append(n+1)
            
        while 1:
            n = randint(0, nmods//2 -1)*2   # 0, 2, 4 or 6			
            n += 1
            if n not in ic:		# Add an A type
                ia.append(n)	
                ib.append(n)	
                break

        for n in range(nmods):
            if len(ib) >= self.Tb:
                break
            if n not in ic+ia: 
                ib.append(n)
                ib.append(n)
        print (ic,ib,ia)
        return ia,ib,ic	

    def selectXE50(self):  # rigid nmods = 8, KU Colleges 50 marks,14-Nov-2024	
        nmods = 8
        ic = []
        ib = []
        ia = []
        n = randint(0, nmods//2 -1)*2   # 0, 2, 4 or 6
        ic.append(n)		# select one C type  
        ib.append(n)		# select one B type  
        ia.append(n)		# select one A type  
        while 1:
            n = randint(0, nmods//2 -1)*2   # 0, 2, 4 or 6			
            if n not in ic:
                ic.append(n)			# Add another C type
                ib.append(n)			# select one B type  
                ia.append(n)			# select one A type  			
                break			

        Csel = [ic[0], ic[0]+1,ic[1],ic[1]+1]
        
        for k in range(nmods):		# Add 4 B and A each
            if k not in Csel: 		
                ib.append(k)
                ia.append(k)

        while 1:		
            n = randint(0, nmods-1)	
            if len(ia) >= self.Ta:
                break
            if n not in Csel and ib.count(n) + ia.count(n) < 3 : 
                ia.append(n)

        print (ic,ib,ia)
        return ia,ib,ic	
        
    def selectXE35(self):  # rigid nmods = 4	
        nmods = 4
        ic = []
        ib = []
        ia = []
        while 1:
            if len(ic) >= self.Tc:
                break
            n = randint(0, nmods-1)			
            if n not in ic and self.ncq[n] != 0:
                ic.append(n)		# Add the C types

        for k in range(nmods):		# Add  B if no C
            if k not in ic:
                ib.append(k)
        for k in range(nmods):		# Add  2 A &  1 B in all
                ib.append(k)
                ia.append(k)
                ia.append(k)
        print(ia,ib,ic)
        return ia,ib,ic	

    def selectXE15(self):  # rigid nmods = 4	
        nmods = 4
        ic = []
        ib = []
        ia = []
        n = randint(0, nmods-1)
        ic.append(n)		# select a C type
        while 1:
            n = randint(0, nmods-1)			
            if n not in ic  and self.ncq[n] != 0:
                ic.append(n)		# Add the second C type
                break

        for k in range(nmods):		# Add  B if no C
            if k not in ic:
                ib.append(k)
                ia.append(k)
        for k in range(nmods):		# Add  2 A &  1 B in all
                ib.append(k)
                ia.append(k)
        print(ia,ib,ic)
        return ia,ib,ic	
    
    #----------- End of QP model routines-----------------

    def makeQP(self):
        #self.log('Entering makeQP. QP Model %s\n'%self.qpModel)
        if self.qpModel == 'ZU70':
            res = self.selectZU70()				
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'ZU50':
            res = self.selectZU50()				
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'YU50':  
            res = self.selectZU50()	# PC uses CU scheme		
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'YU70':
            res = self.selectZU70()	# PC uses CU scheme		
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'AU30':
            res = self.selectAU30()	# PC commerce, botany and CS		
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'BU30':
            res = self.selectBU30()	# PC physics
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'CU30':
            res = self.selectCU30()	# PC  maths		
            if res == False:
                return False
            ia,ib,ic = res	
        # Kannur University
        elif self.qpModel == 'WE50':
            res = self.selectWE50()	# KU dept		
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'XE50':
            res = self.selectXE50()	# KU colleges	
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'XE70':
            res = self.selectXE70()	# KU dept		
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'XE60':
            res = self.selectXE60()	# KU dept		
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'XE35':
            res = self.selectXE35()	# KU dept		
            if res == False:
                return False
            ia,ib,ic = res	
        elif self.qpModel == 'XE15':
            res = self.selectXE15()	# KU dept		
            if res == False:
                return False
            ia,ib,ic = res	
        else:	
            self.log('Error: QP model %s not supported.\n'%self.qpModel)
            return False

        # Verify the required number of questions marked for each type			
        if len(ia) != self.Ta or len(ib) != self.Tb or len(ic) != self.Tc:
            self.log('Failed to find questions for all %d sections\n'%self.numModules)
            return False
        ia.sort()		
        ib.sort()

        nqM1 = (ia+ib+ic).count(0)
        #print('nqM1 = ',nqM1)

        #self.log('ia: %s\nib: %s\nic: %s\n'%(ia, ib, ic))
        self.log('Starting question paper generation:\nSelect C, B and A types by balancing module wise marks.\n')
        self.log('Type C from: ')
        for k in ic:
            self.log('%2d'%(k+1))
        self.log('\n')
        self.log('Type B from: ')
        for k in ib:
            self.log('%2d'%(k+1))
        self.log('\n')
        self.log('Type A from: ')
        for k in ia:
            self.log('%2d'%(k+1))
        self.log('\n')
            
        self.showDist(ia, ib, ic)
        
        #print(ic,ib,ia)
        self.MAXMARK = self.Ma * self.Na + self.Mb * self.Nb + self.Mc * self.Nc
         
        # Filling Question numbers from the selected Modules. 
        usedUnits = []
        for k in range(len(ic)):	# Select C type
            module = ic[k]	
            found = False
            ULIMIT = 1
            if self.qpModel == 'CU30':
                ULIMIT = 5
            for trial in range(30):
                if trial > 25:
                    ULIMIT = 2
                if found == True:
                    break
                if len(self.CQ) == len(ic):
                    break		
                random.shuffle(self.Ctypes[module])		
                for q in self.Ctypes[module]:
                    if q[QUSED] == False and usedUnits.count(q[QUNITN]) < ULIMIT:
                        q[QUSED] = True
                        if q[QUNITN].strip() != '':
                            usedUnits.append(int(q[QUNITN]))					
                        #print(module,'C', q[QSN], q[QUNITN],len(self.CQ),len(ic))
                        self.CQ.append(q)
                        found = True
                        break
                    
        for k in range(len(ib)):	# Select B type
            module = ib[k]	
            found = False
            ULIMIT = 1
            if self.qpModel == 'CU30':
                ULIMIT = 5
            for trial in range(30):
                if trial > 25:
                    ULIMIT = 2
                if found == True:
                    break
                if len(self.BQ) == len(ib):
                    break		
                random.shuffle(self.Btypes[module])		
                for q in self.Btypes[module]:
                    if q[QUSED] == False and usedUnits.count(q[QUNITN]) < ULIMIT:
                        q[QUSED] = True
                        if q[QUNITN].strip() != '':
                            usedUnits.append(int(q[QUNITN]))					
                        #print(module,'B', q[QSN], q[QUNITN],len(self.BQ),len(ib))
                        self.BQ.append(q)
                        found = True
                        break
                        
        for k in range(len(ia)):		# Select A type
            module = ia[k]
            found = False		
            ULIMIT = 1				
            if self.qpModel == 'CU30':
                ULIMIT = 7
            for trial in range(40):	
                if trial > 20:
                    ULIMIT = 2
                if trial > 25:
                    ULIMIT = 4
                if found == True:
                    break
                if len(self.AQ) == len(ia):
                    break		
                random.shuffle(self.Atypes[module])
                for q in self.Atypes[module]:
                    if q[QUSED] == False and usedUnits.count(q[QUNITN]) < ULIMIT :
                        q[QUSED] = True
                        if q[QUNITN].strip() != '':
                            usedUnits.append(int(q[QUNITN]))					
                        #print(module,'A', q[QSN], q[QUNITN])					
                        self.AQ.append(q)
                        found = True
                        break
                        
        #make an interger list of used units for sorting and display
        '''
        iuu = []
        for u in usedUnits:
            iuu.append(int(u))
        '''
        usedUnits.sort()
        print(usedUnits)
        self.log('Select questions, maximum one per unit is desirable but possible only if \n')
        self.log('number of units under a module is proportional to its weightage in marks.\n')
        self.log('Selected from Units: ')
        for u in usedUnits:
            self.log('%s '%u)
        self.log('\n\n')		

        #Verify the results.
        if len(self.AQ) != len(ia):
            self.log('Could get only %d number of A type questions. Requires %d\n'%(len(self.AQ), len(ia)))
            return False
        if len(self.BQ) != len(ib):
            self.log('Could get only %d number of B type questions. Requires %d\n'%(len(self.BQ), len(ib)))
            return False
        if len(self.CQ) != len(ic):
            self.log('Could get only %d number of C type questions. Requires %d\n'%(len(self.CQ), len(ic)))
            return False
        #self.log('Obtained the required number of questions\n')
        return True
        # End of make QP	
    
    #----------------------- LaTex Generation -----------------------
    def texHeader(self):
        tex =  '\\documentclass[12pt, a4paper]{article}\n'
        tex += '\\usepackage{graphicx}\n'
        tex += '\\usepackage{amsmath}\n \\usepackage{amssymb}\n'
        tex += '\\usepackage[scr=rsfs]{mathalpha}\n'
        tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'
        tex += '\\usepackage{fontspec}\n'
        tex += '\\usepackage{enumitem}\n'
        tex += '\\usepackage{multicol}\n'
        tex += '\\usepackage{zref-totpages}\n'
        tex += '\\usepackage{tfrupee}\n'
                
        langbeg = ''
        langend = ''
        tag = 'english'
        if self.language.lower() == 'malayalam':
            tag = 'malayalam'
            langbeg = '\\begin{malayalam}'
            langend = '\\end{malayalam}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{malayalam}\n'
            tex += '\\newfontfamily\\malayalamfont[Script=Malayalam]{Rachana}\n'
            tex += '\\setotherlanguage{hindi}\n'
            tex += '\\newfontfamily\\hindifont[Script=Devanagari]{Lohit Devanagari}\n'			
        elif self.language.lower() == 'hindi':
            tag = 'hindi'	
            langbeg = '\\begin{hindi}\n'
            langend = '\\end{hindi}\n'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{hindi}\n'
            tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
        elif self.language.lower() == 'sanskrit':
            tag = 'hindi'	
            langbeg = '\\begin{hindi}\n' 
            langend = '\\end{hindi}\n'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{hindi}\n'
            tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
        elif self.language.lower() == 'tamil':
            tag = 'tamil'	
            langbeg = '\\begin{tamil}\n'
            langend = '\\end{tamil}\n'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{tamil}\n'
            tex += '\\newfontfamily\\tamilfont[Script=Tamil]{Lohit-Tamil.ttf}\n'
        elif self.language.lower() == 'kannada':
            tag = 'kannada'	
            langbeg = '\\begin{kannada}\n'
            langend = '\\end{kannada}\n'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{kannada}\n'
            tex += '\\newfontfamily\\kannadafont[Script=Kannada]{Lohit-Kannada.ttf}\n'
        elif self.language.lower() == 'arabic':
            tag = 'Arabic'
            langbeg = '\\begin{Arabic}\n'
            langend = '\\end{Arabic}\n'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{arabic}\n'
            tex += '\\setmainfont[Script=Arabic,Scale=1.1]{Scheherazade}\n'	
        elif self.language.lower() == 'urdu':
            tag = 'urdu'
            langbeg = '\\begin{urdu}\n'
            langend = '\\end{urdu}\n'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{urdu}\n'
            tex += '\\newfontfamily\\urdufont[Script=Arabic,Scale=1.0] {NafeesNastaleeq.ttf}\n'
        elif self.language.lower() == 'french':
            tag = 'english'
            langbeg = ''
            langend = ''	
        return tex, tag, langbeg,langend
       
    def sectionHeaders(self):
        if self.qpModel in ['ZU50','ZU70']:  # Calicut University
            self.ha = '\n\\begin{center}\\textbf{Section A}\\end{center}'
            self.ha += '\n\\textbf{May attempt all. Ceiling is %d marks(%d x %d)}\n'%(self.Na*self.Ma, self.Na, self.Ma)
            self.hb = '\n\\begin{center}\\textbf{Section B}\\end{center}'
            self.hb += '\n\\textbf{May attempt all. Ceiling is %d marks(%d x %d)}\n'%(self.Nb*self.Mb, self.Nb, self.Mb)
            self.hc = '\n\\begin{center}\\textbf{Section C}\\end{center}'
            self.hc += '\n\\textbf{Answer any %d question(s)(%d x %d)}\n'%(self.Nc, self.Nc, self.Mc)
        elif self.qpModel in ['YU50','YU70']:  # Providence College
            self.ha = '\n\\begin{center}\\textbf{Section A}\\end{center}'
            self.ha += '\n\\textbf{May attempt all questions. Each carry %d marks. Ceiling %d marks}\n'%(self.Ma, self.Na*self.Ma)
            self.hb = '\n\\begin{center}\\textbf{Section B}\\end{center}'
            self.hb += '\n\\textbf{May attempt all questions. Each carry %d marks. Ceiling %d marks}\n'%(self.Mb, self.Nb*self.Mb)
            self.hc = '\n\\begin{center}\\textbf{Section C}\\end{center}'
            self.hc += '\n\\textbf{Answer any %d question. (%d x %d)marks}\n'%(self.Nc, self.Nc, self.Mc)
        elif self.qpModel == 'AU30':  # Prov. college PG bot,comm,CS
            self.ha = '\n\\begin{center}\\textbf{Section A}\\end{center}'
            self.ha += '\n\\textbf{Answer any %d questions. (%d x %d weightage)}\n'%(self.Na, self.Na, self.Ma)
            self.hb = '\n\\begin{center}\\textbf{Section B}\\end{center}'
            self.hb += '\n\\textbf{Answer any %d questions. (%d x %d weightage)}\n'%(self.Nb, self.Nb, self.Mb)
            self.hc = '\n\\begin{center}\\textbf{Section C}\\end{center}'
            self.hc += '\n\\textbf{Answer any %d question. (%d x %d weightage)}\n'%(self.Nc, self.Nc, self.Mc)
        elif self.qpModel == 'BU30':  # Prov. college PG physics
            self.ha = '\n\\begin{center}\\textbf{Section A}\\end{center}'
            self.ha += '\n\\textbf{Answer all %d questions. (%d x %d weightage)}\n'%(self.Na, self.Na, self.Ma)
            self.hb = '\n\\begin{center}\\textbf{Section B}\\end{center}'
            self.hb += '\n\\textbf{Answer any %d questions. (%d x %d weightage)}\n'%(self.Nb, self.Nb, self.Mb)
            self.hc = '\n\\begin{center}\\textbf{Section C}\\end{center}'
            self.hc += '\n\\textbf{Answer any %d questions. (%d x %d weightage)}\n'%(self.Nc, self.Nc, self.Mc)
        elif self.qpModel == 'CU30':  # Prov. college PG maths
            self.ha = '\n\\begin{center}\\textbf{Section A}\\end{center}'
            self.ha += '\n\\textbf{Answer all %d questions. (%d x %d weightage)}\n'%(self.Na, self.Na, self.Ma)
            self.hb = '\n\\begin{center}\\textbf{Section B}\\end{center}'
            self.hb += '\n\\textbf{Answer any %d questions from each module.(%d x %d weightage)}\n'%(self.Nb/3, self.Nb, self.Mb)
            self.hc = '\n\\begin{center}\\textbf{Section C}\\end{center}'
            self.hc += '\n\\textbf{Answer any %d questions.(%d x %d weightage)}\n'%(self.Nc, self.Nc, self.Mc)
        else:
            self.ha = '\n\\begin{center}\\textbf{Section A}\\end{center}'
            self.ha += '\n\\textbf{Answer any %d questions. Each carry %d marks.}\n'%(self.Na, self.Ma)
            self.hb = '\n\\begin{center}\\textbf{Section B}\\end{center}'
            self.hb += '\n\\textbf{Answer any %d questions. Each carry %d marks.}\n'%(self.Nb, self.Mb)
            self.hc = '\n\\begin{center}\\textbf{Section C}\\end{center}'
            self.hc += '\n\\textbf{Answer any %d questions. Each carry %d marks.}\n'%(self.Nc, self.Mc)
            
    def questionTex(self):
        now = datetime.now() + timedelta(hours = 5, minutes=30)
        self.dt = now.strftime("%d/%m/%Y %H:%M:%S")
        tex,tag,langbeg,langend =  self.texHeader()	
        tex += '\\begin{document}\n'		
        tex += '\\begin{center}\n'
        tex += '\\small %s \\hfill pages: \\ztotpages \\hfill Reg No:....................\n\n'%(self.qpCode)
        tex += '\\hfill  \\small Name :....................  {%s}\n\n'%('')	
        sss = ''
        if self.supple == True:
            sss = 'Suplementary'
        tex += '\\large\n \\textbf {%s Semester %s Degree %s Examination\n'%(self.semester,self.degree,sss)
        tex += '%s %s\\\\\n'%(self.examMonth, self.examYear)
        courseYear = self.courseCode[1:5]
        self.courseName = self.courseName.replace('&','\\&')
        tex += '%s - %s}\n\n'%(self.ccdisp.replace('_','\\_'),self.courseName.replace('_','\\_'))
        tex += '%s Admission onwards\n\n'%courseYear
            
        tex += '\\end{center}\n'
        tex += '\\hrule\n\\vspace{0.05in}\n'
    
        tex += '\\noindent\n' 
        if self.qpModel in ['AU30','BU30','CU30']:
            ss = 'Weightage'
        else:
            ss = 'Marks'
        tex += 'Time : %s hours  \\hfill Maximum %s : %2d\n\\vspace{0.1in}\n\n'%(self.duration, ss, self.MAXMARK)
        
        self.sectionHeaders()
        tex += self.ha        
        
        tex += '\\begin{enumerate}\n'
        tex += langbeg
        for q in self.AQ:
            tex += '\\item %s\n'%(q[QTEXT].strip())
            tex += '\\vspace{0.05in}\n'
        tex += langend
        
        mntext = ['I','II','III']
        if self.qpModel == 'CU30': # Providence Maths
            tex += self.hb
            k = 0
            for q in self.BQ:
                if k%3 == 0:
                    tex += '\n\\textbf{Module %s}\n'%mntext[k//3]
                tex += '\\item %s\n'%(q[QTEXT].strip())
                tex += '\\vspace{0.05in}\n'
                k += 1
        else:
            tex += self.hb
            tex += langbeg            
            for q in self.BQ:
                tex += '\\item %s\n'%(q[QTEXT].strip())
                tex += '\\vspace{0.05in}\n'
            tex += langend
        
        tex += self.hc       
        for q in self.CQ:
            tex += langbeg
            tex += '\\item %s\n'%(q[QTEXT].strip())
            tex += '\\vspace{0.05in}\n'
            tex += langend

        tex += '\\end{enumerate}\n'
        tex += '\\end{document}'

        self.getFigures(tex)
        fn = self.courseCode
        open(fn + '.tex', 'w').write(tex)
        self.answerTex()
        return True

    def answerTex(self):
        now = datetime.now() + timedelta(hours = 5, minutes=30)
        self.dt = now.strftime("%d/%m/%Y %H:%M:%S")
        tex,tag,langbeg,langend =  self.texHeader()	
        tex += '\\begin{document}\n'		
        tex += '\\begin{center}\n'
        
        tex += '\\small %s \\hfill pages: \\ztotpages \\hfill Reg No:....................\n\n'%(self.qpCode)
        tex += '\\hfill  \\small Name :....................  {%s}\n\n'%('')	
        tex += '\\large\n \\textbf {%s Semester FYUG Degree Examination,\n'%self.semester
        tex += '%s %s\\\\\n'%(self.examMonth, self.examYear)
        courseYear = self.courseCode[1:5]
        self.courseName = self.courseName.replace('&','\\&')
        tex += '%s - %s}\n\n'%(self.ccdisp.replace('_','\\_'),self.courseName.replace('_','\\_'))
        tex += '%s Admission onwards\n\n'%courseYear	
        tex += '\\end{center}\n'
        
        tex += 'Answer Keys\n\\hrule\n\\vspace{0.05in}\n'
            
        tex += 'Time : %s hours \\hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%(self.duration, self.MAXMARK)
        tex += '\\hrule\n\\vspace{0.1in}\n'

        tex += '\n\\textbf{Section A (All questions may be attempted. Ceiling is %d marks)}\n'%(self.Na*self.Ma)
                
        tex += '\\begin{enumerate}\n'

        tex += langbeg
        for q in self.AQ:
            muqm = 'M%d:U%s:Q%d:%s\n'%(q[QMODNUM],q[QUNITN],q[QSN],q[QOUT])
            if tag != 'english':
                muqm = '\\begin{english}%s\\end{english}'%muqm			
            tex += '\\item %s \\hfill %s\n'%(q[QTEXT].strip(),muqm)
            if tag == 'english':
                tex += '\nAnswer: \n'
            else:
                tex += '\n\\begin{english}Answer:\\end{english}\n'
            tex += '%s\n'%q[QATEXT].strip()
            tex += '\\vspace{0.05in}\n'
        tex += langend

        tex += '\n\\textbf{Section B (All questions may be attempted. Ceiling is %d marks)}\n'%(self.Nb*self.Mb)

        tex += langbeg
        for q in self.BQ:
            muqm = 'M%d:U%s:Q%d:%s\n'%(q[QMODNUM],q[QUNITN],q[QSN],q[QOUT])
            if tag != 'english':
                muqm = '\\begin{english}%s\\end{english}'%muqm			
            tex += '\\item %s \\hspace*{0pt} \\hfill %s\n'%(q[QTEXT].strip(),muqm)
            if tag == 'english':
                tex += '\nAnswer: \n'
            else:
                tex += '\n\\begin{english}Answer: \\end{english}\n'
            tex += '%s\n'%q[QATEXT].strip()
            tex += '\\vspace{0.05in}\n'
        tex += langend
            
        tex += '\n\\textbf{Section C (Answer any %d questions. Each carries %d marks) }\n'%(self.Nc, self.Mc)
        
        tex += langbeg
        for q in self.CQ:
            muqm = 'M%d:U%s:Q%d:%s\n'%(q[QMODNUM],q[QUNITN],q[QSN],q[QOUT])
            if tag != 'english':
                muqm = '\\begin{english}%s\\end{english}'%muqm			
            tex += '\\item %s \\hspace*{0pt} \\hfill %s\n'%(q[QTEXT].strip(),muqm)
            if tag == 'english':
                tex += '\nAnswer: \n'
            else:
                tex += '\n\\begin{english}Answer: \\end{english}\n'
            tex += '%s\n'%q[QATEXT].strip()
            tex += '\\vspace{0.05in}\n'
        tex += langend
        tex += '\\end{enumerate}\n'
        tex += '\\end{document}'

        fn = self.courseCode + '_ANS'
        open(fn + '.tex', 'w').write(tex)
        return True
        

def callQP(host, myport, db, user, pw, code,year,month,qpcode,ccdisp,sup):
    #print(host,myport,db,user,pw,code,year,month,qpcode,ccdisp)
    if year == '': year = 'Year'
    if month == '': month = 'Month'
    if qpcode == '': qpcode = 'qpcode'
    
    cc = Qpgen(host, myport, db, user, pw,year,month,qpcode, ccdisp,sup)
    if cc.findCourse(code) == False:
        cc.log('Course %s NOT found'%code)
        return False
    cc.courseCode = code
    cc.db = db
    cc.passwd = pw
    if cc.fetchData() != True:
        cc.log('FetchData failed on %s'%cc.courseCode)
        return False
        
    if cc.makeQP() != True:
        cc.log('makeQP failed on %s'%cc.courseCode)
        return False
        
    if cc.questionTex() != True:
        cc.log('Making Question Tex failed on %s'%cc.courseCode)
        return False
        
    if cc.answerTex() != True:
        cc.log('Making Answer Tex failed on %s'%cc.courseCode)
        return False
        
    cc.log('Made the LaTex files. ')
    return True
    
#--------------------------- qpmodule end -----------------

'''
host = sys.argv[1]
port = sys.argv[2]
db = sys.argv[3]
cc = sys.argv[4]
pw = sys.argv[5]
year = sys.argv[6]
month = sys.argv[7]
qpcode = sys.argv[8]
user = cc.lower()
'''

if __name__ == "__main__":
    host = 'scischool.in'
    host = 'localhost'
    port = 5432
    db = 'pcqbank'
    cc = 'Y2024AC_BBA2CJ101'
    
    year = '2025'
    month = 'March'
    qpcode = 'xyz123'
    user = 'reader'
    pw = 'rea123'

    res = callQP(host, port, db, user, pw, cc, year, month, qpcode, cc[8:],'')
    print('result = ', res)
    if res == True:		# PHP expects 0 on success
        sys.exit(0)
    else:
        sys.exit(1)


